@@ -19,6 +19,7 @@
 package io.undertow;
 
 import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.nio.channels.ClosedChannelException;
 import java.nio.file.Path;
 
@@ -26,6 +27,7 @@ import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLHandshakeException;
 import javax.net.ssl.SSLPeerUnverifiedException;
 
+import io.undertow.server.HttpServerExchange;
 import io.undertow.server.RequestTooBigException;
 import io.undertow.server.handlers.form.MultiPartParserDefinition;
 import io.undertow.util.UrlDecodeException;
@@ -85,7 +87,7 @@ public interface UndertowMessages {
     IllegalStateException sessionManagerNotFound();
 
     @Message(id = 13, value = "Argument %s cannot be null")
-    IllegalArgumentException argumentCannotBeNull(final String argument);
+    IllegalArgumentException argumentCannotBeNull(String argument);
 
 //    @Message(id = 14, value = "close() called with data still to be flushed. Please call shutdownWrites() and then call flush() until it returns true before calling close()")
 //    IOException closeCalledWithDataStillToBeFlushed();
@@ -112,13 +114,13 @@ public interface UndertowMessages {
     IllegalArgumentException hashAlgorithmNotFound(String algorithmName);
 
     @Message(id = 23, value = "An invalid Base64 token has been received.")
-    IllegalArgumentException invalidBase64Token(@Cause final IOException cause);
+    IllegalArgumentException invalidBase64Token(@Cause IOException cause);
 
     @Message(id = 24, value = "An invalidly formatted nonce has been received.")
     IllegalArgumentException invalidNonceReceived();
 
     @Message(id = 25, value = "Unexpected token '%s' within header.")
-    IllegalArgumentException unexpectedTokenInHeader(final String name);
+    IllegalArgumentException unexpectedTokenInHeader(String name);
 
     @Message(id = 26, value = "Invalid header received.")
     IllegalArgumentException invalidHeader();
@@ -133,10 +135,10 @@ public interface UndertowMessages {
     IOException chunkedChannelClosedMidChunk();
 
     @Message(id = 30, value = "User %s successfully authenticated.")
-    String userAuthenticated(final String userName);
+    String userAuthenticated(String userName);
 
     @Message(id = 31, value = "User %s has logged out.")
-    String userLoggedOut(final String userName);
+    String userLoggedOut(String userName);
 //
 //    @Message(id = 33, value = "Authentication type %s cannot be combined with %s")
 //    IllegalStateException authTypeCannotBeCombined(String type, String existing);
@@ -154,7 +156,7 @@ public interface UndertowMessages {
     RuntimeException failedToParsePath();
 
     @Message(id = 38, value = "Authentication failed, requested user name '%s'")
-    String authenticationFailed(final String userName);
+    String authenticationFailed(String userName);
 
     @Message(id = 39, value = "Too many query parameters, cannot have more than %s query parameters")
     BadRequestException tooManyQueryParameters(int noParams);
@@ -518,8 +520,8 @@ public interface UndertowMessages {
 //    @Message(id = 159, value = "Max size must be larger than one")
 //    IllegalArgumentException maxSizeMustBeLargerThanOne();
 
-    @Message(id = 161, value = "HTTP/2 header block is too large")
-    String headerBlockTooLarge();
+    @Message(id = 161, value = "HTTP/2 header block is too large, maximum header size is %s")
+    String headerBlockTooLarge(int maxHeaderSize);
 
     @Message(id = 162, value = "An invalid SameSite attribute [%s] is specified. It must be one of %s")
     IllegalArgumentException invalidSameSiteMode(String mode, String validAttributes);
@@ -647,4 +649,10 @@ public interface UndertowMessages {
     @Message(id = 208, value = "Failed to allocate resource")
     IOException failedToAllocateResource();
 
+    @Message(id = 209, value = "Protocol string was too large for the buffer. Either provide a smaller message or a bigger buffer. Protocol: %s")
+    IllegalStateException protocolTooLargeForBuffer(String protocolString);
+
+    @Message(id = 210, value = "Buffer content underflow for exchange '%s', buffer '%s'")
+    IOException bufferUnderflow(HttpServerExchange exchange, ByteBuffer buf);
+
 }
