@@ -30,9 +30,9 @@ import java.nio.channels.FileChannel;
  */
 public interface BufferWritableOutputStream {
 
-    void write(final ByteBuffer[] buffers) throws IOException;
+    void write(ByteBuffer[] buffers) throws IOException;
 
-    void write(final ByteBuffer byteBuffer) throws IOException;
+    void write(ByteBuffer byteBuffer) throws IOException;
 
     void transferFrom(FileChannel source) throws IOException;
 
