@@ -43,6 +43,7 @@ import org.xnio.StreamConnection;
 import org.xnio.channels.StreamSinkChannel;
 import org.xnio.ssl.SslConnection;
 
+import javax.net.ssl.SSLSession;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.Channel;
@@ -52,13 +53,15 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Queue;
 import java.util.Random;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
-import javax.net.ssl.SSLSession;
 
 /**
  * HTTP2 channel.
@@ -67,6 +70,29 @@ import javax.net.ssl.SSLSession;
  */
 public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHttp2StreamSourceChannel, AbstractHttp2StreamSinkChannel> implements Attachable {
 
+    // HTTP2 MAX HEADER SIZE constants, to be properly replaced by UndertowOptions
+    /**
+     * Name of the system property for configuring HTTP2 max header size: {@code "io.undertow.http2-max-header-size"}.
+     * <p>This system property will be replaced by an {@link UndertowOptions Undertow option}.</p>
+     */
+    public static final String HTTP2_MAX_HEADER_SIZE_PROPERTY = "io.undertow.http2-max-header-size";
+    /**
+     * Default value of HTTP2 max header size. If the system property {@code "io.undertow.http2-max-header-size"} is left
+     * undefined, this value will be used for the corresponding {@link #HTTP2_MAX_HEADER_SIZE system property configuration}.
+     */
+    private static final int DEFAULT_HTTP2_MAX_HEADER_SIZE = 20000;
+    /**
+     * System property {@code "io.undertow.http2-max-header-size"} for configuring the max header size configuration for HTTP2
+     * messages.
+     * <p>
+     * The effective maximum size for headers to be used by this channel will be the minimum of the three: this system property,
+     * {@link UndertowOptions#MAX_HEADER_SIZE}, and {@link UndertowOptions#HTTP2_SETTINGS_HEADER_TABLE_SIZE}. When calculating
+     * the minimum, only positive values are taken into account, as values of -1 indicate the absence of a limit.
+     * <p>
+     * To be replaced by an {@link UndertowOptions Undertow option} in a future release.
+     */
+    private static final int HTTP2_MAX_HEADER_SIZE = Integer.getInteger(HTTP2_MAX_HEADER_SIZE_PROPERTY, DEFAULT_HTTP2_MAX_HEADER_SIZE);
+
     public static final String CLEARTEXT_UPGRADE_STRING = "h2c";
 
 
@@ -128,6 +154,9 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     public static final int DEFAULT_MAX_FRAME_SIZE = 16384;
     public static final int MAX_FRAME_SIZE = 16777215;
     public static final int FLOW_CONTROL_MIN_WINDOW = 2;
+    // the time a discarded stream is kept at the stream cache before actually being discarded
+    // (used for handling responses to streams that were closed via rst)
+    private static final int STREAM_CACHE_EVICTION_TIME_MS = 60000;
 
 
     private Http2FrameHeaderParser frameParser;
@@ -148,6 +177,20 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     private final int maxHeaders;
     private final int maxHeaderListSize;
 
+    // the max number of rst frames received per window
+    private final int maxRstFramesPerWindow;
+    // the time window for counting rst frames received
+    private final long rstFramesTimeWindow;
+    // the time in milliseconds the last rst frame was received
+    private long lastReceivedRstFrameMillis = System.currentTimeMillis();
+    // the time in milliseconds the last rst frame was sent
+    private long lastSentRstFrameMillis = System.currentTimeMillis();
+    // the total number of received rst frames during current time windows
+    private int receivedRstFramesPerWindow;
+    // the total number of sent rst frames during current time windows
+    private int sentRstFramesPerWindow;
+
+
     private static final AtomicIntegerFieldUpdater<Http2Channel> sendConcurrentStreamsAtomicUpdater = AtomicIntegerFieldUpdater.newUpdater(
             Http2Channel.class, "sendConcurrentStreams");
 
@@ -198,6 +241,8 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
      */
     private volatile int receiveWindowSize;
 
+    private final StreamCache resetStreamTracker = new StreamCache();
+
 
     public Http2Channel(StreamConnection connectedStreamChannel, String protocol, ByteBufferPool bufferPool, PooledByteBuffer data, boolean clientSide, boolean fromUpgrade, OptionMap settings) {
         this(connectedStreamChannel, protocol, bufferPool, data, clientSide, fromUpgrade, true, null, settings);
@@ -214,7 +259,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         pushEnabled = settings.get(UndertowOptions.HTTP2_SETTINGS_ENABLE_PUSH, true);
         this.initialReceiveWindowSize = settings.get(UndertowOptions.HTTP2_SETTINGS_INITIAL_WINDOW_SIZE, DEFAULT_INITIAL_WINDOW_SIZE);
         this.receiveWindowSize = initialReceiveWindowSize;
-        this.receiveMaxConcurrentStreams = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS, -1);
+        this.receiveMaxConcurrentStreams = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS, UndertowOptions.DEFAULT_HTTP2_SETTINGS_MAX_CONCURRENT_STREAMS);
 
         this.protocol = protocol == null ? Http2OpenListener.HTTP2 : protocol;
         this.maxHeaders = settings.get(UndertowOptions.MAX_HEADERS, clientSide ? -1 : UndertowOptions.DEFAULT_MAX_HEADERS);
@@ -222,12 +267,14 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         encoderHeaderTableSize = settings.get(UndertowOptions.HTTP2_SETTINGS_HEADER_TABLE_SIZE, Hpack.DEFAULT_TABLE_SIZE);
         receiveMaxFrameSize = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_FRAME_SIZE, DEFAULT_MAX_FRAME_SIZE);
         maxPadding = settings.get(UndertowOptions.HTTP2_PADDING_SIZE, 0);
-        maxHeaderListSize = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_HEADER_LIST_SIZE, settings.get(UndertowOptions.MAX_HEADER_SIZE, -1));
+        maxHeaderListSize = getMaxHeaderSize(settings);
         if(maxPadding > 0) {
             paddingRandom = new SecureRandom();
         } else {
             paddingRandom = null;
         }
+        maxRstFramesPerWindow = settings.get(UndertowOptions.MAX_RST_FRAMES_PER_WINDOW, settings.get(UndertowOptions.MAX_RST_FRAMES_PER_WINDOW, UndertowOptions.DEFAULT_MAX_RST_FRAMES_PER_WINDOW));
+        rstFramesTimeWindow = settings.get(UndertowOptions.RST_FRAMES_TIME_WINDOW, settings.get(UndertowOptions.RST_FRAMES_TIME_WINDOW, UndertowOptions.DEFAULT_RST_FRAMES_TIME_WINDOW));
 
         this.decoder = new HpackDecoder(encoderHeaderTableSize);
         this.encoder = new HpackEncoder(encoderHeaderTableSize);
@@ -291,6 +338,42 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         }
     }
 
+    private static int getMaxHeaderSize(OptionMap settings) {
+        final int maxHeaderSizeConfig = settings.get(UndertowOptions.MAX_HEADER_SIZE, HTTP2_MAX_HEADER_SIZE);
+        // soon to be removed http2 settings max header
+        final int http2SettingsMaxHeaderListSize = settings.get(UndertowOptions.HTTP2_SETTINGS_MAX_HEADER_LIST_SIZE, HTTP2_MAX_HEADER_SIZE);
+        if (HTTP2_MAX_HEADER_SIZE > 0) {
+            if (maxHeaderSizeConfig > 0) {
+                if (http2SettingsMaxHeaderListSize > 0) {
+                    return Math.min(Math.min(HTTP2_MAX_HEADER_SIZE, maxHeaderSizeConfig), http2SettingsMaxHeaderListSize);
+                } else {
+                    return Math.min(HTTP2_MAX_HEADER_SIZE, maxHeaderSizeConfig);
+                }
+            } else {
+                if (http2SettingsMaxHeaderListSize > 0) {
+                    return Math.min(HTTP2_MAX_HEADER_SIZE, http2SettingsMaxHeaderListSize);
+                } else {
+                    return HTTP2_MAX_HEADER_SIZE;
+                }
+            }
+        } else {
+            if (maxHeaderSizeConfig > 0) {
+                if (http2SettingsMaxHeaderListSize > 0) {
+                    return Math.min(maxHeaderSizeConfig, http2SettingsMaxHeaderListSize);
+                } else {
+                    return maxHeaderSizeConfig;
+                }
+            } else {
+                if (http2SettingsMaxHeaderListSize > 0) {
+                    return http2SettingsMaxHeaderListSize;
+                } else {
+                    // replace any value <= 0 by -1
+                    return -1;
+                }
+            }
+        }
+    }
+
     private void sendSettings() {
         List<Http2Setting> settings = new ArrayList<>();
         settings.add(new Http2Setting(Http2Setting.SETTINGS_HEADER_TABLE_SIZE, encoderHeaderTableSize));
@@ -390,8 +473,13 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
                     //this is an existing stream
                     //make sure it exists
                     StreamHolder existing = currentStreams.get(frameParser.streamId);
+                    if (existing == null) {
+                        existing = resetStreamTracker.find(frameParser.streamId);
+                    }
                     if(existing == null || existing.sourceClosed) {
-                        sendGoAway(ERROR_PROTOCOL_ERROR);
+                        if (existing != null || resetStreamTracker.find(frameParser.streamId) == null) {
+                            sendGoAway(ERROR_PROTOCOL_ERROR);
+                        }
                         frameData.close();
                         return null;
                     } else if (existing.sourceChannel != null ){
@@ -423,8 +511,13 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
 
                 StreamHolder holder = currentStreams.get(frameParser.streamId);
                 if(holder == null) {
-                    receiveConcurrentStreamsAtomicUpdater.getAndIncrement(this);
-                    currentStreams.put(frameParser.streamId, holder = new StreamHolder((Http2StreamSourceChannel) channel));
+                    holder = resetStreamTracker.find(frameParser.streamId);
+                    if (holder != null) {
+                        holder.sourceChannel = (Http2StreamSourceChannel) channel;
+                    } else {
+                        receiveConcurrentStreamsAtomicUpdater.getAndIncrement(this);
+                        currentStreams.put(frameParser.streamId, holder = new StreamHolder((Http2StreamSourceChannel) channel));
+                    }
                 } else {
                     holder.sourceChannel = (Http2StreamSourceChannel) channel;
                 }
@@ -461,7 +554,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
                     throw new ConnectionErrorException(Http2Channel.ERROR_PROTOCOL_ERROR, UndertowMessages.MESSAGES.streamIdMustNotBeZeroForFrameType(FRAME_TYPE_RST_STREAM));
                 }
                 channel = new Http2RstStreamStreamSourceChannel(this, frameData, parser.getErrorCode(), frameParser.streamId);
-                handleRstStream(frameParser.streamId);
+                handleRstStream(frameParser.streamId, true);
                 if(isIdle(frameParser.streamId)) {
                     sendGoAway(ERROR_PROTOCOL_ERROR);
                 }
@@ -633,8 +726,12 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
 
     @Override
     protected void closeSubChannels() {
+        closeSubChannels(currentStreams);
+        closeSubChannels(resetStreamTracker.getStreamHolders());
+    }
 
-        for (Map.Entry<Integer, StreamHolder> e : currentStreams.entrySet()) {
+    private void closeSubChannels(Map<Integer, StreamHolder> streams) {
+        for (Map.Entry<Integer, StreamHolder> e : streams.entrySet()) {
             StreamHolder holder = e.getValue();
             AbstractHttp2StreamSourceChannel receiver = holder.sourceChannel;
             if(receiver != null) {
@@ -656,7 +753,10 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         List<AbstractFramedStreamSourceChannel<Http2Channel, AbstractHttp2StreamSourceChannel, AbstractHttp2StreamSinkChannel>> channels = new ArrayList<>(currentStreams.size());
         for(Map.Entry<Integer, StreamHolder> entry : currentStreams.entrySet()) {
             if(!entry.getValue().sourceClosed) {
-                channels.add(entry.getValue().sourceChannel);
+                final Http2StreamSourceChannel sourceChannel = entry.getValue().sourceChannel;
+                if (sourceChannel != null) {
+                    channels.add(sourceChannel);
+                }
             }
         }
         return channels;
@@ -763,7 +863,7 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
             StreamHolder holder = currentStreams.get(streamId);
             Http2StreamSinkChannel stream = holder != null ? holder.sinkChannel : null;
             if (stream == null) {
-                if(isIdle(streamId)) {
+                if (resetStreamTracker.find(streamId) == null && isIdle(streamId)) {
                     sendGoAway(ERROR_PROTOCOL_ERROR);
                 }
             } else {
@@ -899,6 +999,10 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         return http2SynStreamStreamSinkChannel;
     }
 
+    public String getProtocolRequestId() {
+        return Integer.toString(streamIdCounter);
+    }
+
     /**
      * Adds a received pushed stream into the current streams for a client. The
      * stream is added into the currentStream and lastAssignedStreamOtherSide is incremented.
@@ -1111,19 +1215,24 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
             //no point sending if the channel is closed
             return;
         }
-        handleRstStream(streamId);
-        if(UndertowLogger.REQUEST_IO_LOGGER.isDebugEnabled()) {
-            UndertowLogger.REQUEST_IO_LOGGER.debugf(new ClosedChannelException(), "Sending rststream on channel %s stream %s", this, streamId);
+        handleRstStream(streamId, false);
+        if (!this.isThisGoneAway()) {
+            if (UndertowLogger.REQUEST_IO_LOGGER.isDebugEnabled()) {
+                UndertowLogger.REQUEST_IO_LOGGER.debugf(new ClosedChannelException(),
+                        "Sending rststream on channel %s stream %s", this, streamId);
+            }
+            Http2RstStreamSinkChannel channel = new Http2RstStreamSinkChannel(this, streamId, statusCode);
+            flushChannelIgnoreFailure(channel);
         }
-        Http2RstStreamSinkChannel channel = new Http2RstStreamSinkChannel(this, streamId, statusCode);
-        flushChannelIgnoreFailure(channel);
     }
 
-    private void handleRstStream(int streamId) {
-        StreamHolder holder = currentStreams.remove(streamId);
+    private StreamHolder handleRstStream(int streamId, boolean receivedRst) {
+        final StreamHolder holder = currentStreams.remove(streamId);
         if(holder != null) {
+            resetStreamTracker.store(streamId, holder);
             if(streamId % 2 == (isClient() ? 1 : 0)) {
                 sendConcurrentStreamsAtomicUpdater.getAndDecrement(this);
+                holder.sent = true;
             } else {
                 receiveConcurrentStreamsAtomicUpdater.getAndDecrement(this);
             }
@@ -1133,6 +1242,68 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
             if (holder.sourceChannel != null) {
                 holder.sourceChannel.rstStream();
             }
+            if (receivedRst) {
+                if (holder.sinkChannel != null && holder.sourceChannel == null) {
+                    //Server side originated, no input from client other than RST
+                    //this can happen on page refresh when push happens, but client
+                    //still has valid cache entry
+                    //NOTE: this is specific case when its set.
+                    holder.resetByPeer = receivedRst;
+                } else {
+                    trackReceivedRstWindow();
+                }
+            }
+            else {
+                trackSentRstWindow();
+            }
+        } else if(receivedRst){
+            final StreamHolder resetStream = resetStreamTracker.find(streamId, true);
+            if(resetStream != null && resetStream.resetByPeer) {
+                //This means other side reset stream at some point.
+                //depending on peer or network latency our frames might be late and
+                //cause other end to flare up with RST, this RST can be safely ignored.
+                //TODO: do we need to check error code?
+            } else {
+                trackReceivedRstWindow();
+            }
+        } else {
+            trackSentRstWindow();
+        }
+        return holder;
+    }
+
+    private void trackReceivedRstWindow() {
+        long currentTimeMillis = System.currentTimeMillis();
+        // reset the window tracking
+        if (currentTimeMillis - lastReceivedRstFrameMillis >= rstFramesTimeWindow) {
+            lastReceivedRstFrameMillis = currentTimeMillis;
+            receivedRstFramesPerWindow = 1;
+        } else {
+            receivedRstFramesPerWindow++;
+            if (receivedRstFramesPerWindow > maxRstFramesPerWindow) {
+                sendGoAway(Http2Channel.ERROR_ENHANCE_YOUR_CALM);
+                UndertowLogger.REQUEST_IO_LOGGER.debugf(
+                        "Reached maximum number of rst frames %s during %s ms, sending GO_AWAY 11",
+                        maxRstFramesPerWindow, rstFramesTimeWindow);
+            }
+        }
+    }
+
+    private void trackSentRstWindow() {
+        long currentTimeMillis = System.currentTimeMillis();
+        // reset the window tracking
+        if (currentTimeMillis - lastSentRstFrameMillis >= rstFramesTimeWindow) {
+            lastSentRstFrameMillis = currentTimeMillis;
+            sentRstFramesPerWindow = 1;
+        } else {
+            sentRstFramesPerWindow++;
+            if (sentRstFramesPerWindow > maxRstFramesPerWindow) {
+                sendGoAway(Http2Channel.ERROR_ENHANCE_YOUR_CALM);
+                UndertowLogger.REQUEST_IO_LOGGER.debugf(
+                        "Reached maximum number of sent rst frames %s during %s ms, sending GO_AWAY 11",
+                        maxRstFramesPerWindow, rstFramesTimeWindow);
+                IoUtils.safeClose(this);
+            }
         }
     }
 
@@ -1169,8 +1340,9 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
 
     Http2StreamSourceChannel removeStreamSource(int streamId) {
         StreamHolder existing = currentStreams.get(streamId);
-        if(existing == null){
-            return null;
+        if (existing == null) {
+            existing = resetStreamTracker.find(streamId);
+            return existing == null? null : existing.sourceChannel;
         }
         existing.sourceClosed = true;
         Http2StreamSourceChannel ret = existing.sourceChannel;
@@ -1189,7 +1361,10 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     Http2StreamSourceChannel getIncomingStream(int streamId) {
         StreamHolder existing = currentStreams.get(streamId);
         if(existing == null){
-            return null;
+            existing = resetStreamTracker.find(streamId);
+            if (existing == null) {
+                return null;
+            }
         }
         return existing.sourceChannel;
     }
@@ -1231,6 +1406,14 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
     private static final class StreamHolder {
         boolean sourceClosed = false;
         boolean sinkClosed = false;
+        /**
+         * This flag is set only in case of short lived server push that was reset by remote end.
+         */
+        boolean resetByPeer = false;
+        /**
+         * flag indicate whether our side originated. This is done for caching purposes, handlng differs.
+         */
+        boolean sent = false;
         Http2StreamSourceChannel sourceChannel;
         Http2StreamSinkChannel sinkChannel;
 
@@ -1241,5 +1424,86 @@ public class Http2Channel extends AbstractFramedChannel<Http2Channel, AbstractHt
         StreamHolder(Http2StreamSinkChannel sinkChannel) {
             this.sinkChannel = sinkChannel;
         }
+
+        @Override
+        public String toString() {
+            return "StreamHolder [sourceClosed=" + sourceClosed + ", sinkClosed=" + sinkClosed + ", resetByPeer=" + resetByPeer
+                    + ", sent=" + sent + ", sourceChannel=" + sourceChannel + ", sinkChannel=" + sinkChannel + "]";
+        }
+
+    }
+
+    // cache that keeps track of streams until they can be evicted @see Http2Channel#RST_STREAM_EVICATION_TIME
+    private static final class StreamCache {
+        private Map<Integer, StreamHolder> streamHolders = new ConcurrentHashMap<>();
+        // entries are sorted per creation time
+        private Queue<StreamCacheEntry> entries = new ConcurrentLinkedQueue<>();
+
+        private void store(int streamId, StreamHolder streamHolder) {
+            if (streamHolder == null) {
+                return;
+            }
+            streamHolders.put(streamId, streamHolder);
+            entries.add(new StreamCacheEntry(streamId));
+        }
+
+        /**
+         * Method will return only sent
+         * @param streamId
+         * @return
+         */
+        private StreamHolder find(final int streamId) {
+            return find(streamId, false);
+        }
+
+        private StreamHolder find(final int streamId, final boolean all) {
+            for (Iterator<StreamCacheEntry> iterator = entries.iterator(); iterator.hasNext();) {
+                StreamCacheEntry entry = iterator.next();
+                if (entry.shouldEvict()) {
+                    iterator.remove();
+                    StreamHolder holder = streamHolders.remove(entry.streamId);
+                    if(!holder.sent || holder.resetByPeer) {
+                        //if its not our end of chain, its just cached, so we only cache for purpose of
+                        // handling eager RST
+                        continue;
+                    }
+                    AbstractHttp2StreamSourceChannel receiver = holder.sourceChannel;
+                    if(receiver != null) {
+                        IoUtils.safeClose(receiver);
+                    }
+                    Http2StreamSinkChannel sink = holder.sinkChannel;
+                    if(sink != null) {
+                        if (sink.isWritesShutdown()) {
+                            ChannelListeners.invokeChannelListener(sink.getIoThread(), sink, ((ChannelListener.SimpleSetter) sink.getWriteSetter()).get());
+                        }
+                        IoUtils.safeClose(sink);
+                    }
+                } else break;
+            }
+            final StreamHolder holder = streamHolders.get(streamId);
+            if(holder != null && (!all && !holder.sent)) {
+                return null;
+            } else {
+                return holder;
+            }
+        }
+
+        private Map<Integer, StreamHolder> getStreamHolders() {
+            return streamHolders;
+        }
+    }
+
+    private static class StreamCacheEntry {
+        int streamId;
+        long time;
+
+        StreamCacheEntry(int streamId) {
+            this.streamId = streamId;
+            this.time = System.currentTimeMillis();
+        }
+
+        public boolean shouldEvict() {
+            return System.currentTimeMillis() - time > STREAM_CACHE_EVICTION_TIME_MS;
+        }
     }
 }
