@@ -46,6 +46,7 @@ abstract class Http2HeaderBlockParser extends Http2PushBackParser implements Hpa
 
     private final HpackDecoder decoder;
     private int frameRemaining = -1;
+    private int totalHeaderLength = 0;
     private boolean invalid = false;
     private boolean processingPseudoHeaders = true;
     private final boolean client;
@@ -81,7 +82,7 @@ abstract class Http2HeaderBlockParser extends Http2PushBackParser implements Hpa
     protected void handleData(ByteBuffer resource, Http2FrameHeaderParser header) throws IOException {
         boolean continuationFramesComing = Bits.anyAreClear(header.flags, Http2Channel.HEADERS_FLAG_END_HEADERS);
         if (frameRemaining == -1) {
-            frameRemaining = header.length;
+            frameRemaining = totalHeaderLength = header.length;
         }
         final boolean moreDataThisFrame = resource.remaining() < frameRemaining;
         final int pos = resource.position();
@@ -156,7 +157,7 @@ abstract class Http2HeaderBlockParser extends Http2PushBackParser implements Hpa
         if(maxHeaderListSize > 0) {
             headerSize += (name.length() + value.length() + 32);
             if (headerSize > maxHeaderListSize) {
-                throw new HpackException(UndertowMessages.MESSAGES.headerBlockTooLarge(), Http2Channel.ERROR_PROTOCOL_ERROR);
+                throw new HpackException(UndertowMessages.MESSAGES.headerBlockTooLarge(maxHeaderListSize), Http2Channel.ERROR_PROTOCOL_ERROR);
             }
         }
         if(maxHeaders > 0 && headerMap.size() > maxHeaders) {
@@ -200,9 +201,15 @@ abstract class Http2HeaderBlockParser extends Http2PushBackParser implements Hpa
     }
     protected abstract int getPaddingLength();
     @Override
-    protected void moreData(int data) {
-        super.moreData(data);
+    protected boolean moreData(int data) {
+        boolean acceptMoreData = super.moreData(data);
         frameRemaining += data;
+        totalHeaderLength += data;
+        if (maxHeaderListSize > 0 && totalHeaderLength > maxHeaderListSize) {
+            UndertowLogger.REQUEST_LOGGER.debug(UndertowMessages.MESSAGES.headerBlockTooLarge(maxHeaderListSize));
+            return false;
+        }
+        return acceptMoreData;
     }
 
     public boolean isInvalid() {
