@@ -63,12 +63,23 @@ public class BasicSSLSessionInfo implements SSLSessionInfo {
             ByteArrayInputStream stream = new ByteArrayInputStream(certificateBytes);
             Collection<? extends java.security.cert.Certificate> certCol = cf.generateCertificates(stream);
             this.peerCertificate = new java.security.cert.Certificate[certCol.size()];
-            this.certificate = new X509Certificate[certCol.size()];
+            X509Certificate[] legacyCertificate = new X509Certificate[certCol.size()];
             int i=0;
             for(java.security.cert.Certificate cert : certCol) {
                 this.peerCertificate[i] = cert;
-                this.certificate[i++] = X509Certificate.getInstance(cert.getEncoded());
+                if (legacyCertificate != null) {
+                    try {
+                        legacyCertificate[i] = X509Certificate.getInstance(cert.getEncoded());
+                    } catch (CertificateException ce) {
+                        // [UNDERTOW-1969] We don't care about deprecated JDK methods failure caused by the fact newer JDKs
+                        // doesn't support them anymore. "this.certificate" is used only by deprecated method
+                        // {@link SSLSessionInfo.getPeerCertificateChain()} which call should be avoided by API users.
+                        legacyCertificate = null;
+                    }
+                }
+                i++;
             }
+            this.certificate = legacyCertificate;
         } else {
             this.peerCertificate = null;
             this.certificate = null;
@@ -138,12 +149,13 @@ public class BasicSSLSessionInfo implements SSLSessionInfo {
 
     @Override
     public java.security.cert.Certificate[] getPeerCertificates() throws SSLPeerUnverifiedException {
-        if (certificate == null) {
+        if (peerCertificate == null) {
             throw UndertowMessages.MESSAGES.peerUnverified();
         }
         return peerCertificate;
     }
 
+    @Deprecated
     @Override
     public X509Certificate[] getPeerCertificateChain() throws SSLPeerUnverifiedException {
         if (certificate == null) {
