@@ -1,28 +1,37 @@
 package io.undertow.server;
 
+import io.undertow.UndertowLogger;
+import sun.misc.Unsafe;
+
 import java.lang.reflect.Field;
+import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.nio.ByteBuffer;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
-
-import io.undertow.UndertowLogger;
-import sun.misc.Unsafe;
+import java.util.LinkedList;
+import java.util.Queue;
 
 /**
  * {@link DirectByteBufferDeallocator} Utility class used to free direct buffer memory.
  */
 public final class DirectByteBufferDeallocator {
+    private static final int DEALLOCATION_DELAY_MILLIS = 100;
     private static final boolean SUPPORTED;
     private static final Method cleaner;
     private static final Method cleanerClean;
+    private static final ThreadLocal<Queue<QueuedByteBuffer>> bufferQueue;
 
     private static final Unsafe UNSAFE;
 
 
     static {
+        bufferQueue = ThreadLocal.withInitial(()-> new LinkedList());
         String versionString = System.getProperty("java.specification.version");
-        if(versionString.startsWith("1.")) {
+        if(versionString.equals("0.9")) {
+            //android hardcoded
+            versionString = "11";
+        } else if(versionString.startsWith("1.")) {
             versionString = versionString.substring(2);
         }
         int version = Integer.parseInt(versionString);
@@ -70,19 +79,45 @@ public final class DirectByteBufferDeallocator {
     public static void free(ByteBuffer buffer) {
         if (SUPPORTED && buffer != null && buffer.isDirect()) {
             try {
-                if (UNSAFE != null) {
-                    //use the JDK9 method
-                    cleanerClean.invoke(UNSAFE, buffer);
-                } else {
-                    Object cleaner = DirectByteBufferDeallocator.cleaner.invoke(buffer);
-                    cleanerClean.invoke(cleaner);
+                // queue
+                final Queue<QueuedByteBuffer> queuedByteBuffers = bufferQueue.get();
+                // only clean buffers that have been returned DEALLOCATION_DELAY_MIILLIS ms ago
+                final long targetTimeMillis = System.currentTimeMillis() - DEALLOCATION_DELAY_MILLIS;
+                QueuedByteBuffer queuedByteBuffer = queuedByteBuffers.peek();
+                while (queuedByteBuffer != null) {
+                    if (queuedByteBuffer.timeStamp > targetTimeMillis) {
+                        break;
+                    }
+                    queuedByteBuffers.remove();
+                    cleanBuffer(queuedByteBuffer.byteBuffer);
+                    queuedByteBuffer = queuedByteBuffers.peek();
                 }
+                // put the buffer to be cleaned in the queue
+                // the goal here is to create a delay to make sure
+                // that the buffer is not immediately deallocated
+                // as there is a small window of time in which the
+                // buffer is still accessible via local variables;
+                // if a direct buffer is cleaned and then written to
+                // or read from, the behavior of the sdk is unpredictable
+                queuedByteBuffers.add(new QueuedByteBuffer(buffer));
             } catch (Throwable t) {
                 UndertowLogger.ROOT_LOGGER.directBufferDeallocationFailed(t);
             }
         }
     }
 
+    private static void cleanBuffer(ByteBuffer buffer) throws InvocationTargetException, IllegalAccessException {
+        if (buffer != null) {
+            if (UNSAFE != null) {
+                //use the JDK9 method
+                cleanerClean.invoke(UNSAFE, buffer);
+            } else {
+                Object cleaner = DirectByteBufferDeallocator.cleaner.invoke(buffer);
+                cleanerClean.invoke(cleaner);
+            }
+        }
+    }
+
     private static Unsafe getUnsafe() {
         if (System.getSecurityManager() != null) {
             return AccessController.doPrivileged(new PrivilegedAction<Unsafe>() {
@@ -147,4 +182,14 @@ public final class DirectByteBufferDeallocator {
             throw new RuntimeException("JDK did not allow accessing method", t);
         }
     }
+
+    private static class QueuedByteBuffer {
+        final long timeStamp;
+        final ByteBuffer byteBuffer;
+
+        QueuedByteBuffer(ByteBuffer byteBuffer) {
+            this.timeStamp = System.currentTimeMillis();
+            this.byteBuffer = byteBuffer;
+        }
+    }
 }
