@@ -22,18 +22,29 @@ package io.undertow.server;
  * Listener interface for events that are run at the completion of a request/response
  * cycle (i.e. when the request has been completely read, and the response has been fully written).
  *
- * At this point it is to late to modify the exchange further.
+ * At this point it is too late to modify the exchange further.
  *
- * Completion listeners are invoked in reverse order,
+ * Implementations are required invoke {@link NextListener#proceed()} to allow other listeners to release
+ * resources, even in failure scenarios. This chain allows transfer of request ownership between
+ * listeners in order to complete using non-blocking mechanisms, and must not be used to conditionally
+ * proceed.
+ *
+ * Completion listeners are invoked in reverse order.
  *
  * @author Stuart Douglas
  */
 public interface ExchangeCompletionListener {
 
-    void exchangeEvent(final HttpServerExchange exchange, final NextListener nextListener);
+    void exchangeEvent(HttpServerExchange exchange, NextListener nextListener);
 
     interface NextListener {
 
+        /**
+         * Invokes the next {@link ExchangeCompletionListener listener}. This must be executed by
+         * every {@link ExchangeCompletionListener} implementation, and may be invoked from a
+         * different thread as a callback. Failure to proceed will cause resource leaks, and
+         * potentially cause requests to hang.
+         */
         void proceed();
 
     }
