@@ -268,6 +268,7 @@ public class IPAddressAccessControlHandler implements HttpHandler {
             int no = Integer.parseInt(part);
             prefix |= no;
         }
+        prefix &= mask;
         ipv4acl.add(new PrefixIpV4PeerMatch(deny, peer, mask, prefix));
     }
 
@@ -348,7 +349,7 @@ public class IPAddressAccessControlHandler implements HttpHandler {
             this.pattern = pattern;
         }
 
-        abstract boolean matches(final InetAddress address);
+        abstract boolean matches(InetAddress address);
 
         boolean isDeny() {
             return deny;
@@ -377,7 +378,7 @@ public class IPAddressAccessControlHandler implements HttpHandler {
         }
 
         @Override
-        boolean matches(final InetAddress address) {
+        boolean matches(InetAddress address) {
             return Arrays.equals(address.getAddress(), this.address);
         }
     }
