@@ -62,7 +62,7 @@ public interface ModClusterStatus {
         /**
          * @deprecated Use {@link LoadBalancer#getMaxRetries()}.
          */
-        @Deprecated
+        @Deprecated(since="2.0.4", forRemoval=true)
         int getMaxAttempts();
     }
 
