@@ -16,6 +16,7 @@ import java.io.IOException;
 import java.nio.file.FileSystem;
 import java.nio.file.FileSystems;
 import java.nio.file.Files;
+import java.nio.file.LinkOption;
 import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -133,7 +134,16 @@ public class PathResourceManager implements ResourceManager  {
             throw UndertowMessages.MESSAGES.argumentCannotBeNull("base");
         }
         this.fileSystem = builder.base.getFileSystem();
-        String basePath = builder.base.normalize().toAbsolutePath().toString();
+        String basePath = null;
+        try {
+            if(builder.followLinks) {
+                basePath = builder.base.normalize().toRealPath().toString();
+            } else {
+                basePath = builder.base.normalize().toRealPath(LinkOption.NOFOLLOW_LINKS).toString();
+            }
+        } catch (IOException e) {
+            throw UndertowMessages.MESSAGES.failedToInitializePathManager(builder.base.toString(), e);
+        }
         if (!basePath.endsWith(fileSystem.getSeparator())) {
             basePath = basePath + fileSystem.getSeparator();
         }
@@ -186,6 +196,9 @@ public class PathResourceManager implements ResourceManager  {
     }
 
     public Resource getResource(final String p) {
+        if( p == null ) {
+            return null;
+        }
         String path;
         //base always ends with a /
         if (p.startsWith("/")) {
@@ -230,9 +243,15 @@ public class PathResourceManager implements ResourceManager  {
                 log.tracef("Failed to get path resource %s from path resource manager with base %s, as the path did not exist", p, base);
                 return null;
             }
-        } catch (Exception e) {
+        } catch (IOException e) {
             UndertowLogger.REQUEST_LOGGER.debugf(e, "Invalid path %s", p);
             return null;
+        } catch (SecurityException e) {
+            UndertowLogger.REQUEST_LOGGER.errorf(e, "Missing JSM permissions for path %s", p);
+            throw e;
+        } catch (Exception e) {
+            UndertowLogger.REQUEST_LOGGER.debugf(e, "Other issue for path %s", p);
+            return null;
         }
     }
 
@@ -300,13 +319,13 @@ public class PathResourceManager implements ResourceManager  {
     /**
      * Returns true is some element of path inside base path is a symlink.
      */
-    private SymlinkResult getSymlinkBase(final String base, final Path file) throws IOException {
+    private SymlinkResult getSymlinkBase(final String base, final Path file) {
         int nameCount = file.getNameCount();
         Path root = fileSystem.getPath(base);
         int rootCount = root.getNameCount();
         Path f = file;
         for (int i = nameCount - 1; i>=0; i--) {
-            if (Files.isSymbolicLink(f)) {
+            if (SecurityActions.isSymbolicLink(f)) {
                 return new SymlinkResult(i+1 > rootCount, f);
             }
             f = f.getParent();
