@@ -430,6 +430,9 @@ public abstract class AbstractFramedChannel<C extends AbstractFramedChannel<C, R
                         readData = null;
                     }
                     if(frameDataRemaining == 0) {
+                        if (isLastFrameReceived()) {
+                            forceFree = true;
+                        }
                         receiver = null;
                     }
                     return null;
@@ -494,6 +497,12 @@ public abstract class AbstractFramedChannel<C extends AbstractFramedChannel<C, R
                 partialRead = true;
             }
             return null;
+        } catch (IllegalStateException e) {
+            if (!pooled.isFreed()) {
+                throw e;
+            }
+            channel.close();
+            return null;
         } catch (IOException|RuntimeException|Error e) {
             //something has code wrong with parsing, close the read side
             //we don't close the write side, as the underlying implementation will most likely want to send an error
@@ -503,28 +512,45 @@ public abstract class AbstractFramedChannel<C extends AbstractFramedChannel<C, R
         }finally {
             //if the receive caused the channel to break the close listener may be have been called
             //which will make readData null
-            if (readData != null) {
-                if (!pooled.getBuffer().hasRemaining() || forceFree) {
-                    if(pooled.getBuffer().capacity() < 1024 || forceFree) {
-                        //if there is less than 1k left we don't allow it to be re-aquired
-                        readData = null;
+            if (readData != null || pooled != null) {
+                if (pooled.isOpen()) {
+                    try {
+                        if (forceFree || !pooled.getBuffer().hasRemaining()) {
+                            if (forceFree || pooled.getBuffer().capacity() < 1024) {
+                                //if there is less than 1k left we don't allow it to be re-aquired
+                                readData = null;
+                            }
+                            //even though this is freed we may un-free it if we get a new packet
+                            //this prevents many small reads resulting in a large number of allocated buffers
+                            pooled.close();
+                        }
+                    } catch (IllegalStateException illegalStateException) {
+                        // ignore, it can happen if we are racing with a close that has closed the pooled
                     }
-                    //even though this is freed we may un-free it if we get a new packet
-                    //this prevents many small reads resulting in a large number of allocated buffers
-                    pooled.close();
-
                 }
             }
             if(requiresReinvoke) {
+                ByteBuffer buffer = null;
                 if(pooled != null && !pooled.isFreed()) {
-                    if(pooled.getBuffer().remaining() == reinvokeDataRemaining) {
-                        pooled.close();
-                        readData = null;
-                        UndertowLogger.REQUEST_IO_LOGGER.debugf("Partial message read before connection close %s", this);
+                    try {
+                        buffer = pooled.getBuffer();
+                    } catch (IllegalStateException e) {
+                        if (!pooled.isFreed()) {
+                            throw e;
+                        }
                     }
                 }
+                if (buffer != null && buffer.remaining() == reinvokeDataRemaining) {
+                    pooled.close();
+                    readData = null;
+                    UndertowLogger.REQUEST_IO_LOGGER.debugf("Partial message read before connection close %s", this);
+                }
                 channel.getSourceChannel().wakeupReads();
             }
+            // race condition, asynchronous close while reading
+            if (channel.isWriteShutdown() && channel.isReadShutdown()) {
+                pooled.close();
+            }
         }
     }
 
@@ -568,8 +594,13 @@ public abstract class AbstractFramedChannel<C extends AbstractFramedChannel<C, R
                 }
             } while (!outstandingBuffersUpdater.compareAndSet(this, expect, expect + 1));
         }
-        PooledByteBuffer buf = bufferPool.allocate();
-        return this.readData = new ReferenceCountedPooled(buf, 1, maxQueuedBuffers > 0 ? freeNotifier : null);
+        try {
+            PooledByteBuffer buf = bufferPool.allocate();
+            return this.readData = new ReferenceCountedPooled(buf, 1, maxQueuedBuffers > 0 ? freeNotifier : null);
+        } catch (IllegalStateException e) {
+            // pool is closed
+            return null;
+        }
     }
 
     /**
@@ -1028,7 +1059,17 @@ public abstract class AbstractFramedChannel<C extends AbstractFramedChannel<C, R
                     @Override
                     public void run() {
                         while (localReadData != null  && !localReadData.isFreed()) {
-                            int rem = localReadData.getBuffer().remaining();
+                            final ByteBuffer buffer;
+                            try {
+                                buffer = localReadData.getBuffer();
+                            } catch (IllegalStateException e) {
+                                if (localReadData.isFreed()) {
+                                    return;
+                                } else {
+                                    throw e;
+                                }
+                            }
+                            int rem = buffer.remaining();
                             ChannelListener listener = receiveSetter.get();
                             if(listener == null) {
                                 listener = DRAIN_LISTENER;
