@@ -372,11 +372,11 @@ public abstract class HttpRequestParser {
         //TODO: list type headers?
     }
 
-    abstract void handleHttpVerb(ByteBuffer buffer, final ParseState currentState, final HttpServerExchange builder) throws BadRequestException;
+    abstract void handleHttpVerb(ByteBuffer buffer, ParseState currentState, HttpServerExchange builder) throws BadRequestException;
 
-    abstract void handleHttpVersion(ByteBuffer buffer, final ParseState currentState, final HttpServerExchange builder) throws BadRequestException;
+    abstract void handleHttpVersion(ByteBuffer buffer, ParseState currentState, HttpServerExchange builder) throws BadRequestException;
 
-    abstract void handleHeader(ByteBuffer buffer, final ParseState currentState, final HttpServerExchange builder) throws BadRequestException;
+    abstract void handleHeader(ByteBuffer buffer, ParseState currentState, HttpServerExchange builder) throws BadRequestException;
 
     /**
      * The parse states for parsing the path.
@@ -388,6 +388,10 @@ public abstract class HttpRequestParser {
     private static final int IN_PATH = 4;
     private static final int HOST_DONE = 5;
 
+    private static final int PATH_SEGMENT_START = 0;
+    private static final int PATH_DOT_SEGMENT = 1;
+    private static final int PATH_NON_DOT_SEGMENT = 2;
+
     /**
      * Parses a path value
      *
@@ -403,6 +407,8 @@ public abstract class HttpRequestParser {
         int canonicalPathStart = state.pos;
         boolean urlDecodeRequired = state.urlDecodeRequired;
 
+        int pathSubState = 0;
+
         while (buffer.hasRemaining()) {
             char next = (char) (buffer.get() & 0xFF);
             if(!allowUnescapedCharactersInUrl && !ALLOWED_TARGET_CHARACTER[next]) {
@@ -426,6 +432,11 @@ public abstract class HttpRequestParser {
                 state.urlDecodeRequired = urlDecodeRequired;
                 // store at canonical path the partial path parsed up until here
                 state.canonicalPath.append(stringBuilder.substring(canonicalPathStart));
+                if (parseState == IN_PATH && pathSubState == PATH_DOT_SEGMENT) {
+                    // Inside a dot-segment (".", ".."), we don't want to allow removal of the ';' character from
+                    // the path. This is to avoid path traversal issues - "/..;" should not be treated as "/..".
+                    state.canonicalPath.append(";");
+                }
                 state.stringBuilder.append(";");
                 // set position to end of path (possibly start of parameter name)
                 state.pos = state.stringBuilder.length();
@@ -459,6 +470,18 @@ public abstract class HttpRequestParser {
                 } else if (next == '/' && parseState != HOST_DONE) {
                     parseState = IN_PATH;
                 }
+
+                // This is helper state that tracks if the parser is currently in a path dot-segment (".", "..") or not.
+                if (parseState == IN_PATH) {
+                    if (next == '/') {
+                        pathSubState = PATH_SEGMENT_START;
+                    } else if (next == '.' && (pathSubState == PATH_SEGMENT_START || pathSubState == PATH_DOT_SEGMENT)) {
+                        pathSubState = PATH_DOT_SEGMENT;
+                    } else {
+                        pathSubState = PATH_NON_DOT_SEGMENT;
+                    }
+                }
+
                 stringBuilder.append(next);
             }
 
@@ -474,10 +497,15 @@ public abstract class HttpRequestParser {
             exchange.setRelativePath("/");
             exchange.setRequestURI(path, true);
         } else if (parseState < HOST_DONE && state.canonicalPath.length() == 0) {
-            String decodedPath = decode(path, urlDecodeRequired, state, slashDecodingFlag, false);
-            exchange.setRequestPath(decodedPath);
-            exchange.setRelativePath(decodedPath);
-            exchange.setRequestURI(path, false);
+            final String decodedRequestPath = decode(path, urlDecodeRequired, state, slashDecodingFlag, false);
+            exchange.setRequestPath(decodedRequestPath);
+            exchange.setRelativePath(decodedRequestPath);
+            if(urlDecodeRequired && allowUnescapedCharactersInUrl) {
+                final String uri = decode(path, urlDecodeRequired, state, slashDecodingFlag, false);
+                exchange.setRequestURI(uri);
+            } else {
+                exchange.setRequestURI(path);
+            }
         } else {
             handleFullUrl(state, exchange, canonicalPathStart, urlDecodeRequired, path, parseState);
         }
@@ -497,10 +525,15 @@ public abstract class HttpRequestParser {
 
     private void handleFullUrl(ParseState state, HttpServerExchange exchange, int canonicalPathStart, boolean urlDecodeRequired, String path, int parseState) {
         state.canonicalPath.append(path.substring(canonicalPathStart));
-        String thePath = decode(state.canonicalPath.toString(), urlDecodeRequired, state, slashDecodingFlag, false);
-        exchange.setRequestPath(thePath);
-        exchange.setRelativePath(thePath);
-        exchange.setRequestURI(path, parseState == HOST_DONE);
+        final String requestPath = decode(state.canonicalPath.toString(), urlDecodeRequired, state, slashDecodingFlag, false);
+        exchange.setRequestPath(requestPath);
+        exchange.setRelativePath(requestPath);
+        if(urlDecodeRequired && allowUnescapedCharactersInUrl) {
+            final String uri = decode(path, urlDecodeRequired, state, slashDecodingFlag, false);
+            exchange.setRequestURI(uri, parseState == HOST_DONE);
+        } else {
+            exchange.setRequestURI(path, parseState == HOST_DONE);
+        }
     }
 
 
@@ -514,10 +547,11 @@ public abstract class HttpRequestParser {
      */
     @SuppressWarnings("unused")
     final void handleQueryParameters(ByteBuffer buffer, ParseState state, HttpServerExchange exchange) throws BadRequestException {
-        StringBuilder stringBuilder = state.stringBuilder;
+        final StringBuilder stringBuilder = state.stringBuilder;
         int queryParamPos = state.pos;
         int mapCount = state.mapCount;
         boolean urlDecodeRequired = state.urlDecodeRequired;
+        boolean nextQueryParamDecodeRequired = state.urlDecodeRequired;
         String nextQueryParam = state.nextQueryParam;
 
         //so this is a bit funky, because it not only deals with parsing, but
@@ -528,19 +562,23 @@ public abstract class HttpRequestParser {
         //we encounter an encoded character
 
         while (buffer.hasRemaining()) {
-            char next = (char) (buffer.get() & 0xFF);
+            final char next = (char) (buffer.get() & 0xFF);
             if(!allowUnescapedCharactersInUrl && !ALLOWED_TARGET_CHARACTER[next]) {
                 throw new BadRequestException(UndertowMessages.MESSAGES.invalidCharacterInRequestTarget(next));
             }
             if (next == ' ' || next == '\t') {
-                final String queryString = stringBuilder.toString();
+                String queryString = stringBuilder.toString();
+                if(urlDecodeRequired && this.allowUnescapedCharactersInUrl) {
+                    exchange.setNonDecodedQueryString(queryString);
+                    queryString = decode(queryString, urlDecodeRequired, state, slashDecodingFlag, false);
+                }
                 exchange.setQueryString(queryString);
                 if (nextQueryParam == null) {
                     if (queryParamPos != stringBuilder.length()) {
-                        exchange.addQueryParam(decode(stringBuilder.substring(queryParamPos), urlDecodeRequired, state, true, true), "");
+                        exchange.addQueryParam(decode(stringBuilder.substring(queryParamPos), nextQueryParamDecodeRequired, state, true, true), "");
                     }
                 } else {
-                    exchange.addQueryParam(nextQueryParam, decode(stringBuilder.substring(queryParamPos), urlDecodeRequired, state, true, true));
+                    exchange.addQueryParam(nextQueryParam, decode(stringBuilder.substring(queryParamPos), nextQueryParamDecodeRequired, state, true, true));
                 }
                 state.state = ParseState.VERSION;
                 state.stringBuilder.setLength(0);
@@ -553,26 +591,26 @@ public abstract class HttpRequestParser {
                 throw UndertowMessages.MESSAGES.failedToParsePath();
             } else {
                 if (decode && (next == '+' || next == '%' || next > 127)) { //+ is only a whitespace substitute in the query part of the URL
-                    urlDecodeRequired = true;
+                    nextQueryParamDecodeRequired = urlDecodeRequired = true;
                 } else if (next == '=' && nextQueryParam == null) {
-                    nextQueryParam = decode(stringBuilder.substring(queryParamPos), urlDecodeRequired, state, true, true);
-                    urlDecodeRequired = false;
+                    nextQueryParam = decode(stringBuilder.substring(queryParamPos), nextQueryParamDecodeRequired, state, true, true);
+                    nextQueryParamDecodeRequired = false;
                     queryParamPos = stringBuilder.length() + 1;
                 } else if (next == '&' && nextQueryParam == null) {
                     if (++mapCount >= maxParameters) {
                         throw UndertowMessages.MESSAGES.tooManyQueryParameters(maxParameters);
                     }
                     if (queryParamPos != stringBuilder.length()) {
-                        exchange.addQueryParam(decode(stringBuilder.substring(queryParamPos), urlDecodeRequired, state, true, true), "");
+                        exchange.addQueryParam(decode(stringBuilder.substring(queryParamPos), nextQueryParamDecodeRequired, state, true, true), "");
                     }
-                    urlDecodeRequired = false;
+                    nextQueryParamDecodeRequired = false;
                     queryParamPos = stringBuilder.length() + 1;
                 } else if (next == '&') {
                     if (++mapCount >= maxParameters) {
                         throw UndertowMessages.MESSAGES.tooManyQueryParameters(maxParameters);
                     }
-                    exchange.addQueryParam(nextQueryParam, decode(stringBuilder.substring(queryParamPos), urlDecodeRequired, state, true, true));
-                    urlDecodeRequired = false;
+                    exchange.addQueryParam(nextQueryParam, decode(stringBuilder.substring(queryParamPos), nextQueryParamDecodeRequired, state, true, true));
+                    nextQueryParamDecodeRequired = false;
                     queryParamPos = stringBuilder.length() + 1;
                     nextQueryParam = null;
                 }
