@@ -18,7 +18,10 @@
 
 package io.undertow.server.protocol.http;
 
+import java.io.IOException;
+
 import io.undertow.UndertowLogger;
+import io.undertow.UndertowMessages;
 import io.undertow.UndertowOptions;
 import io.undertow.conduits.ChunkedStreamSinkConduit;
 import io.undertow.conduits.ChunkedStreamSourceConduit;
@@ -61,7 +64,7 @@ class HttpTransferEncoding {
     private HttpTransferEncoding() {
     }
 
-    public static void setupRequest(final HttpServerExchange exchange) {
+    public static void setupRequest(final HttpServerExchange exchange) throws IOException {
         final HeaderMap requestHeaders = exchange.getRequestHeaders();
         final String connectionHeader = requestHeaders.getFirst(Headers.CONNECTION);
         final String transferEncodingHeader = requestHeaders.getLast(Headers.TRANSFER_ENCODING);
@@ -103,7 +106,7 @@ class HttpTransferEncoding {
 
     }
 
-    private static boolean handleRequestEncoding(final HttpServerExchange exchange, String transferEncodingHeader, String contentLengthHeader, HttpServerConnection connection, PipeliningBufferingStreamSinkConduit pipeliningBuffer, boolean persistentConnection) {
+    private static boolean handleRequestEncoding(final HttpServerExchange exchange, String transferEncodingHeader, String contentLengthHeader, HttpServerConnection connection, PipeliningBufferingStreamSinkConduit pipeliningBuffer, boolean persistentConnection) throws IOException {
 
         HttpString transferEncoding = Headers.IDENTITY;
         if (transferEncodingHeader != null) {
@@ -120,9 +123,15 @@ class HttpTransferEncoding {
                 // no content - immediately start the next request, returning an empty stream for this one
                 Connectors.terminateRequest(exchange);
             } else {
-                // fixed-length content - add a wrapper for a fixed-length stream
-                ConduitStreamSourceChannel sourceChannel = ((HttpServerConnection) exchange.getConnection()).getChannel().getSourceChannel();
-                sourceChannel.setConduit(fixedLengthStreamSourceConduitWrapper(contentLength, sourceChannel.getConduit(), exchange));
+                if (exchange.getMaxEntitySize() > 0 && exchange.getMaxEntitySize() < contentLength && exchange.isResponseChannelAvailable()) {
+                    persistentConnection = false;
+                    Connectors.terminateRequest(exchange);
+                    throw UndertowMessages.MESSAGES.requestEntityWasTooLarge(exchange.getMaxEntitySize());
+                } else {
+                    // fixed-length content - add a wrapper for a fixed-length stream
+                    ConduitStreamSourceChannel sourceChannel = ((HttpServerConnection) exchange.getConnection()).getChannel().getSourceChannel();
+                    sourceChannel.setConduit(fixedLengthStreamSourceConduitWrapper(contentLength, sourceChannel.getConduit(), exchange));
+                }
             }
         } else if (transferEncodingHeader != null) {
             //identity transfer encoding
@@ -155,7 +164,7 @@ class HttpTransferEncoding {
             return !(connectionHeader != null && Headers.CLOSE.equalToString(connectionHeader));
         } else if (exchange.isHttp10()) {
             if (connectionHeader != null) {
-                if (Headers.KEEP_ALIVE.equals(new HttpString(connectionHeader))) {
+                if (Headers.KEEP_ALIVE.equalToString(connectionHeader)) {
                     return true;
                 }
             }
