@@ -49,6 +49,7 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
     private final UndertowXnioSsl ssl;
     private final ByteBufferPool bufferPool;
     private final OptionMap sslOptionMap;
+    private final StringBuilder stringBuilder = new StringBuilder();
 
     private int byteCount;
     private String protocol;
@@ -56,7 +57,6 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
     private InetAddress destAddress;
     private int sourcePort = -1;
     private int destPort = -1;
-    private StringBuilder stringBuilder = new StringBuilder();
     private boolean carriageReturnSeen = false;
     private boolean parsingUnknown = false;
 
@@ -103,7 +103,7 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
         } catch (IOException e) {
             UndertowLogger.REQUEST_IO_LOGGER.ioException(e);
             IoUtils.safeClose(streamConnection);
-        } catch (Exception e) {
+        } catch (Throwable e) {
             UndertowLogger.REQUEST_IO_LOGGER.ioException(new IOException(e));
             IoUtils.safeClose(streamConnection);
         } finally {
@@ -115,7 +115,7 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
 
 
 
-    private void parseProxyProtocolV2(PooledByteBuffer buffer, AtomicBoolean freeBuffer) throws Exception {
+    private void parseProxyProtocolV2(PooledByteBuffer buffer, AtomicBoolean freeBuffer) throws IOException {
         while (byteCount < SIG.length) {
             byte c = buffer.getBuffer().get();
 
@@ -222,7 +222,7 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
         return;
     }
 
-    private void parseProxyProtocolV1(PooledByteBuffer buffer, AtomicBoolean freeBuffer) throws Exception {
+    private void parseProxyProtocolV1(PooledByteBuffer buffer, AtomicBoolean freeBuffer) throws IOException {
         while (buffer.getBuffer().hasRemaining()) {
             char c = (char) buffer.getBuffer().get();
             if (byteCount < NAME.length) {
@@ -281,31 +281,46 @@ class ProxyProtocolReadListener implements ChannelListener<StreamSourceChannel>
                                 throw UndertowMessages.MESSAGES.invalidProxyHeader();
                             }
                         } else if (sourceAddress == null) {
-                            sourceAddress = parseAddress(stringBuilder.toString(), protocol);
-                            stringBuilder.setLength(0);
+                            try {
+                                sourceAddress = parseAddress(stringBuilder.toString(), protocol);
+                            } finally {
+                                stringBuilder.setLength(0);
+                            }
                         } else if (destAddress == null) {
-                            destAddress = parseAddress(stringBuilder.toString(), protocol);
-                            stringBuilder.setLength(0);
+                            try {
+                                destAddress = parseAddress(stringBuilder.toString(), protocol);
+                            } finally {
+                                stringBuilder.setLength(0);
+                            }
                         } else {
-                            sourcePort = Integer.parseInt(stringBuilder.toString());
-                            stringBuilder.setLength(0);
+                            try {
+                                sourcePort = Integer.parseInt(stringBuilder.toString());
+                            } finally {
+                                stringBuilder.setLength(0);
+                            }
                         }
                         break;
                     case '\r':
                         if (destPort == -1 && sourcePort != -1 && !carriageReturnSeen && stringBuilder.length() > 0) {
-                            destPort = Integer.parseInt(stringBuilder.toString());
-                            stringBuilder.setLength(0);
+                            try {
+                                destPort = Integer.parseInt(stringBuilder.toString());
+                            } finally {
+                                stringBuilder.setLength(0);
+                            }
                             carriageReturnSeen = true;
                         } else if (protocol == null) {
                             if (UNKNOWN.equals(stringBuilder.toString())) {
                                 parsingUnknown = true;
                                 carriageReturnSeen = true;
                             }
+                            stringBuilder.setLength(0);
                         } else {
+                            stringBuilder.setLength(0);
                             throw UndertowMessages.MESSAGES.invalidProxyHeader();
                         }
                         break;
                     case '\n':
+                        stringBuilder.setLength(0);
                         throw UndertowMessages.MESSAGES.invalidProxyHeader();
                     default:
                         stringBuilder.append(c);
