@@ -175,8 +175,10 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
         String sessionID = config.findSessionId(serverExchange);
         final SessionImpl session = new SessionImpl(this, config, serverExchange.getIoThread(), serverExchange.getConnection().getWorker(), defaultSessionTimeout);
         if (sessionID != null) {
-            if (!saveSessionID(sessionID, session))
+            if (!saveSessionID(sessionID, session)) {
+                session.destroy();
                 throw UndertowMessages.MESSAGES.sessionWithIdAlreadyExists(sessionID);
+            }
             // else: succeeded to use requested session id
         } else {
             sessionID = createAndSaveNewID(session);
@@ -226,15 +228,18 @@ public class InMemorySessionManager implements SessionManager, SessionManagerSta
     public Session getSession(final HttpServerExchange serverExchange, final SessionConfig config) {
         if (serverExchange != null) {
             SessionImpl newSession = serverExchange.getAttachment(NEW_SESSION);
-            if(newSession != null) {
+            if (newSession != null) {
                 return newSession;
             }
-        } else {
-            return null;
         }
+
+        if (config == null) {
+            throw UndertowMessages.MESSAGES.couldNotFindSessionCookieConfig();
+        }
+
         String sessionId = config.findSessionId(serverExchange);
         InMemorySessionManager.SessionImpl session = (SessionImpl) getSession(sessionId);
-        if(session != null && serverExchange != null) {
+        if (session != null && serverExchange != null) {
             session.requestStarted(serverExchange);
         }
         return session;
