@@ -17,6 +17,7 @@
  */
 package io.undertow.websockets.core;
 
+import io.undertow.UndertowLogger;
 import io.undertow.conduits.IdleTimeoutConduit;
 import io.undertow.server.protocol.framed.AbstractFramedChannel;
 import io.undertow.server.protocol.framed.AbstractFramedStreamSourceChannel;
@@ -24,9 +25,12 @@ import io.undertow.server.protocol.framed.FrameHeaderData;
 import io.undertow.websockets.extensions.ExtensionFunction;
 import org.xnio.ChannelExceptionHandler;
 import org.xnio.ChannelListener;
+import org.xnio.ChannelListener.SimpleSetter;
 import org.xnio.ChannelListeners;
 import org.xnio.IoUtils;
 import org.xnio.OptionMap;
+import org.xnio.Options;
+
 import io.undertow.connector.ByteBufferPool;
 import io.undertow.connector.PooledByteBuffer;
 import org.xnio.StreamConnection;
@@ -49,7 +53,16 @@ import java.util.Set;
  * @author Stuart Douglas
  */
 public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketChannel, StreamSourceFrameChannel, StreamSinkFrameChannel> {
-
+    /**
+     * Configure a read timeout for a web socket, in milliseconds. If its present it will override {@link org.xnio.Options.READ_TIMEOUT}. If the given amount of time elapses without
+     * a successful read taking place, the socket's next read will throw a {@link ReadTimeoutException}.
+     */
+    public static final String WEB_SOCKETS_READ_TIMEOUT = "io.undertow.websockets.core.read-timeout";
+    /**
+     * Configure a write timeout for a web socket, in milliseconds. If its present it will override {@link org.xnio.Options.WRITE_TIMEOUT}. If the given amount of time elapses without
+     * a successful write taking place, the socket's next write will throw a {@link WriteTimeoutException}.
+     */
+    public static final String WEB_SOCKETS_WRITE_TIMEOUT = "io.undertow.websockets.core.write-timeout";
     private final boolean client;
 
     private final WebSocketVersion version;
@@ -82,6 +95,7 @@ public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketCh
      */
     private final Set<WebSocketChannel> peerConnections;
 
+    private static final CloseMessage CLOSE_MSG = new CloseMessage(CloseMessage.GOING_AWAY, WebSocketMessages.MESSAGES.messageCloseWebSocket());
     /**
      * Create a new {@link WebSocketChannel}
      * 8
@@ -104,6 +118,22 @@ public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketCh
         this.hasReservedOpCode = extensionFunction.hasExtensionOpCode();
         this.subProtocol = subProtocol;
         this.peerConnections = peerConnections;
+        final String webSocketReadTimeout = System.getProperty(WEB_SOCKETS_READ_TIMEOUT);
+        if(webSocketReadTimeout != null) {
+            try {
+                this.setOption(Options.READ_TIMEOUT, Integer.parseInt(webSocketReadTimeout));
+            } catch (Exception e) {
+                UndertowLogger.ROOT_LOGGER.failedToSetWSTimeout(e);
+            }
+        }
+        final String webSocketWriteTimeout = System.getProperty(WEB_SOCKETS_WRITE_TIMEOUT);
+        if(webSocketWriteTimeout != null) {
+            try {
+                this.setOption(Options.WRITE_TIMEOUT, Integer.parseInt(webSocketWriteTimeout));
+            } catch (Exception e) {
+                UndertowLogger.ROOT_LOGGER.failedToSetWSTimeout(e);
+            }
+        }
         addCloseTask(new ChannelListener<WebSocketChannel>() {
             @Override
             public void handleEvent(WebSocketChannel channel) {
@@ -147,7 +177,7 @@ public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketCh
     }
 
     @Override
-    protected synchronized void lastDataRead() {
+    protected void lastDataRead() {
         if(!closeFrameReceived && !closeFrameSent) {
             //the peer has likely already gone away, but try and send a close frame anyway
             //this will likely just result in the write() failing an immediate connection termination
@@ -158,6 +188,15 @@ public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketCh
             } catch (IOException e) {
                 IoUtils.safeClose(this);
             }
+            final ChannelListener<?> listener = ((SimpleSetter<WebSocketChannel>)getReceiveSetter()).get();
+            if(listener instanceof AbstractReceiveListener) {
+                final AbstractReceiveListener abstractReceiveListener = (AbstractReceiveListener) listener;
+                try {
+                    abstractReceiveListener.onCloseMessage(CLOSE_MSG, this);
+                } catch(Exception e) {
+                    e.printStackTrace();
+                }
+            }
         }
     }
 
@@ -415,7 +454,7 @@ public abstract class WebSocketChannel extends AbstractFramedChannel<WebSocketCh
         /**
          * @return The channel, or null if the channel is not available yet
          */
-        StreamSourceFrameChannel getChannel(final PooledByteBuffer data);
+        StreamSourceFrameChannel getChannel(PooledByteBuffer data);
 
         /**
          * Handles the data, any remaining data will be pushed back
