@@ -30,12 +30,15 @@ import java.util.HashMap;
 import java.util.Map;
 
 import org.apache.commons.codec.digest.DigestUtils;
+import org.apache.http.HttpEntity;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.HttpPost;
+import org.apache.http.entity.ContentType;
 import org.apache.http.entity.StringEntity;
 import org.apache.http.entity.mime.FormBodyPart;
 import org.apache.http.entity.mime.HttpMultipartMode;
 import org.apache.http.entity.mime.MultipartEntity;
+import org.apache.http.entity.mime.MultipartEntityBuilder;
 import org.apache.http.entity.mime.content.FileBody;
 import org.apache.http.entity.mime.content.StringBody;
 import org.junit.Assert;
@@ -46,6 +49,7 @@ import org.xnio.IoUtils;
 import io.undertow.server.HttpHandler;
 import io.undertow.server.HttpServerExchange;
 import io.undertow.server.handlers.BlockingHandler;
+import io.undertow.server.handlers.form.MultiPartParserDefinition.FileTooLargeException;
 import io.undertow.testutils.DefaultServer;
 import io.undertow.testutils.HttpClientUtils;
 import io.undertow.testutils.TestHttpClient;
@@ -195,33 +199,44 @@ public class MultipartFormDataParserTestCase {
             client.getConnectionManager().shutdown();
         }
     }
-
     private static HttpHandler createInMemoryReadingHandler(final long fileSizeThreshold) {
+        return createInMemoryReadingHandler(fileSizeThreshold, -1, null);
+    }
+
+    private static HttpHandler createInMemoryReadingHandler(final long fileSizeThreshold, final long maxInvidualFileThreshold, final HttpHandler async) {
         return new HttpHandler() {
             @Override
             public void handleRequest(final HttpServerExchange exchange) throws Exception {
                 MultiPartParserDefinition multiPartParserDefinition = new MultiPartParserDefinition();
                 multiPartParserDefinition.setFileSizeThreshold(fileSizeThreshold);
+                multiPartParserDefinition.setMaxIndividualFileSize(maxInvidualFileThreshold);
                 final FormDataParser parser = FormParserFactory.builder(false)
                         .addParsers(new FormEncodedDataDefinition(), multiPartParserDefinition)
                         .build().createParser(exchange);
-                try {
-                    FormData data = parser.parseBlocking();
-                    exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR);
-                    if (data.getFirst("formValue").getValue().equals("myValue")) {
-                        FormData.FormValue file = data.getFirst("file");
-                        if (file.isFileItem()) {
-                            exchange.setStatusCode(StatusCodes.OK);
-                            logResult(exchange, file.getFileItem().isInMemory(), file.getFileName(), stream2String(file));
+                if (async == null) {
+                    try {
+                        FormData data = parser.parseBlocking();
+                        exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR);
+                        if (data.getFirst("formValue").getValue().equals("myValue")) {
+                            FormData.FormValue file = data.getFirst("file");
+                            if (file.isFileItem()) {
+                                exchange.setStatusCode(StatusCodes.OK);
+                                logResult(exchange, file.getFileItem().isInMemory(), file.getFileName(), stream2String(file));
+                            }
                         }
+                        exchange.endExchange();
+                    } catch (FileTooLargeException e) {
+                        exchange.setStatusCode(StatusCodes.REQUEST_ENTITY_TOO_LARGE);
+                        exchange.endExchange();
+                    } catch (Throwable e) {
+                        e.printStackTrace();
+                        exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR);
+                        exchange.endExchange();
+                    } finally {
+                        IoUtils.safeClose(parser);
                     }
-                    exchange.endExchange();
-                } catch (Throwable e) {
-                    e.printStackTrace();
-                    exchange.setStatusCode(StatusCodes.INTERNAL_SERVER_ERROR);
-                    exchange.endExchange();
-                } finally {
-                    IoUtils.safeClose(parser);
+                } else {
+                    parser.parse(async);
                 }
             }
 
@@ -340,6 +355,86 @@ public class MultipartFormDataParserTestCase {
         }
     }
 
+    @Test
+    public void testLargeContentWithoutFileNameWithSmallFileSizeThreshold() throws Exception {
+        DefaultServer.setRootHandler(new BlockingHandler(createInMemoryReadingHandler(10)));
+        File file = new File("tmp_upload_file.txt");
+        try (TestHttpClient client = new TestHttpClient()) {
+            file.createNewFile();
+            // 32 kb content, which exceeds default fieldSizeThreshold, the FormData.FormValue will be a FileItem
+            writeLargeFileContent(file, 0x4000 * 2);
+            HttpPost post = new HttpPost(DefaultServer.getDefaultServerURL() + "/path");
+            HttpEntity entity = MultipartEntityBuilder.create()
+                    .addTextBody("formValue", "myValue", ContentType.TEXT_PLAIN)
+                    .addBinaryBody("file", Files.newInputStream(file.toPath()))
+                    .build();
+            post.setEntity(entity);
+            HttpResponse result = client.execute(post);
+            Assert.assertEquals(StatusCodes.OK, result.getStatusLine().getStatusCode());
+            String resp = HttpClientUtils.readResponse(result);
+
+            Map<String, String> parsedResponse = parse(resp);
+
+            Assert.assertEquals("false", parsedResponse.get("in_memory"));
+            Assert.assertEquals(DigestUtils.md5Hex(Files.newInputStream(file.toPath())), parsedResponse.get("hash"));
+            Assert.assertEquals(parsedResponse.get("file_name"), "null");
+        } finally {
+            if (!file.delete()) {
+                file.deleteOnExit();
+            }
+        }
+    }
+
+    @Test
+    public void testFileUploadWithFileSizeThresholdOverflow_Sync() throws Exception {
+        DefaultServer.setRootHandler(new BlockingHandler(createInMemoryReadingHandler(10, 1, null)));
+
+        TestHttpClient client = new TestHttpClient();
+        try {
+
+            HttpPost post = new HttpPost(DefaultServer.getDefaultServerURL() + "/path");
+            MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
+
+            entity.addPart("formValue", new StringBody("myValue", "text/plain", StandardCharsets.UTF_8));
+            entity.addPart("file", new FileBody(new File(MultipartFormDataParserTestCase.class.getResource("uploadfile.txt").getFile())));
+
+            post.setEntity(entity);
+            HttpResponse result = client.execute(post);
+            Assert.assertEquals(StatusCodes.REQUEST_ENTITY_TOO_LARGE, result.getStatusLine().getStatusCode());
+
+        } finally {
+            client.getConnectionManager().shutdown();
+        }
+    }
+
+    @Test
+    public void testFileUploadWithFileSizeThresholdOverflow_ASync() throws Exception {
+        DefaultServer.setRootHandler(new BlockingHandler(createInMemoryReadingHandler(10, 1, new HttpHandler() {
+
+            @Override
+            public void handleRequest(HttpServerExchange exchange) throws Exception {
+                throw new Exception();
+            }
+        })));
+
+        TestHttpClient client = new TestHttpClient();
+        try {
+
+            HttpPost post = new HttpPost(DefaultServer.getDefaultServerURL() + "/path");
+            MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
+
+            entity.addPart("formValue", new StringBody("myValue", "text/plain", StandardCharsets.UTF_8));
+            entity.addPart("file", new FileBody(new File(MultipartFormDataParserTestCase.class.getResource("uploadfile.txt").getFile())));
+
+            post.setEntity(entity);
+            HttpResponse result = client.execute(post);
+            Assert.assertEquals(StatusCodes.REQUEST_ENTITY_TOO_LARGE, result.getStatusLine().getStatusCode());
+
+        } finally {
+            client.getConnectionManager().shutdown();
+        }
+    }
+
     private void writeLargeFileContent(File file, int size) throws IOException {
         int textLength = "content".getBytes().length;
         FileOutputStream fos = new FileOutputStream(file);
