@@ -40,6 +40,6 @@ import jakarta.servlet.ServletContext;
  */
 public interface ServletExtension {
 
-    void handleDeployment(final DeploymentInfo deploymentInfo, final ServletContext servletContext);
+    void handleDeployment(DeploymentInfo deploymentInfo, ServletContext servletContext);
 
 }
