@@ -27,6 +27,6 @@ import io.undertow.server.session.SessionManager;
  */
 public interface SessionManagerFactory {
 
-    SessionManager createSessionManager(final Deployment deployment);
+    SessionManager createSessionManager(Deployment deployment);
 
 }
