@@ -20,24 +20,24 @@ package io.undertow.servlet.core;
 
 import io.undertow.servlet.UndertowServletLogger;
 
-import javax.servlet.ServletContext;
-import javax.servlet.ServletContextAttributeEvent;
-import javax.servlet.ServletContextAttributeListener;
-import javax.servlet.ServletContextEvent;
-import javax.servlet.ServletContextListener;
-import javax.servlet.ServletException;
-import javax.servlet.ServletRequest;
-import javax.servlet.ServletRequestAttributeEvent;
-import javax.servlet.ServletRequestAttributeListener;
-import javax.servlet.ServletRequestEvent;
-import javax.servlet.ServletRequestListener;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpSession;
-import javax.servlet.http.HttpSessionAttributeListener;
-import javax.servlet.http.HttpSessionBindingEvent;
-import javax.servlet.http.HttpSessionEvent;
-import javax.servlet.http.HttpSessionIdListener;
-import javax.servlet.http.HttpSessionListener;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletContextAttributeEvent;
+import jakarta.servlet.ServletContextAttributeListener;
+import jakarta.servlet.ServletContextEvent;
+import jakarta.servlet.ServletContextListener;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.ServletRequest;
+import jakarta.servlet.ServletRequestAttributeEvent;
+import jakarta.servlet.ServletRequestAttributeListener;
+import jakarta.servlet.ServletRequestEvent;
+import jakarta.servlet.ServletRequestListener;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpSession;
+import jakarta.servlet.http.HttpSessionAttributeListener;
+import jakarta.servlet.http.HttpSessionBindingEvent;
+import jakarta.servlet.http.HttpSessionEvent;
+import jakarta.servlet.http.HttpSessionIdListener;
+import jakarta.servlet.http.HttpSessionListener;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -63,8 +63,8 @@ public class ApplicationListeners implements Lifecycle {
             ServletContextAttributeListener.class,
             ServletRequestListener.class,
             ServletRequestAttributeListener.class,
-            javax.servlet.http.HttpSessionListener.class,
-            javax.servlet.http.HttpSessionAttributeListener.class,
+            jakarta.servlet.http.HttpSessionListener.class,
+            jakarta.servlet.http.HttpSessionAttributeListener.class,
             HttpSessionIdListener.class};
 
     private static final ThreadLocal<ListenerState> IN_PROGRAMATIC_SC_LISTENER_INVOCATION = new ThreadLocal<ListenerState>() {
@@ -212,7 +212,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletContextAttributeEvent sre = new ServletContextAttributeEvent(servletContext, name, value);
         for (int i = 0; i < servletContextAttributeListeners.length; ++i) {
-            this.<ServletContextAttributeListener>get(servletContextAttributeListeners[i]).attributeAdded(sre);
+            ManagedListener listener = servletContextAttributeListeners[i];
+            try {
+                this.<ServletContextAttributeListener> get(listener).attributeAdded(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeAdded", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -222,7 +227,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletContextAttributeEvent sre = new ServletContextAttributeEvent(servletContext, name, value);
         for (int i = 0; i < servletContextAttributeListeners.length; ++i) {
-            this.<ServletContextAttributeListener>get(servletContextAttributeListeners[i]).attributeRemoved(sre);
+            ManagedListener listener = servletContextAttributeListeners[i];
+            try {
+                this.<ServletContextAttributeListener> get(listener).attributeRemoved(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeRemoved", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -232,7 +242,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletContextAttributeEvent sre = new ServletContextAttributeEvent(servletContext, name, value);
         for (int i = 0; i < servletContextAttributeListeners.length; ++i) {
-            this.<ServletContextAttributeListener>get(servletContextAttributeListeners[i]).attributeReplaced(sre);
+            ManagedListener listener = servletContextAttributeListeners[i];
+            try {
+                this.<ServletContextAttributeListener> get(listener).attributeReplaced(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeReplaced", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -250,10 +265,11 @@ public class ApplicationListeners implements Lifecycle {
             } catch (RuntimeException e) {
                 UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("requestInitialized", servletRequestListeners[i].getListenerInfo().getListenerClass(), e);
                 for (; i >= 0; i--) {
+                    ManagedListener listener = servletRequestListeners[i];
                     try {
-                        this.<ServletRequestListener>get(servletRequestListeners[i]).requestDestroyed(sre);
+                        this.<ServletRequestListener> get(listener).requestDestroyed(sre);
                     } catch (Throwable t) {
-                        UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("requestDestroyed", servletRequestListeners[i].getListenerInfo().getListenerClass(), e);
+                        UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("requestDestroyed", listener.getListenerInfo().getListenerClass(), e);
                     }
                 }
                 throw e;
@@ -284,7 +300,13 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletRequestAttributeEvent sre = new ServletRequestAttributeEvent(servletContext, request, name, value);
         for (int i = 0; i < servletRequestAttributeListeners.length; ++i) {
-            this.<ServletRequestAttributeListener>get(servletRequestAttributeListeners[i]).attributeAdded(sre);
+            ManagedListener listener = servletRequestAttributeListeners[i];
+            try {
+                this.<ServletRequestAttributeListener> get(listener).attributeAdded(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeAdded", listener.getListenerInfo().getListenerClass(), e);
+            }
+
         }
     }
 
@@ -294,7 +316,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletRequestAttributeEvent sre = new ServletRequestAttributeEvent(servletContext, request, name, value);
         for (int i = 0; i < servletRequestAttributeListeners.length; ++i) {
-            this.<ServletRequestAttributeListener>get(servletRequestAttributeListeners[i]).attributeRemoved(sre);
+            ManagedListener listener = servletRequestAttributeListeners[i];
+            try {
+                this.<ServletRequestAttributeListener> get(listener).attributeRemoved(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeRemoved", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -304,7 +331,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final ServletRequestAttributeEvent sre = new ServletRequestAttributeEvent(servletContext, request, name, value);
         for (int i = 0; i < servletRequestAttributeListeners.length; ++i) {
-            this.<ServletRequestAttributeListener>get(servletRequestAttributeListeners[i]).attributeReplaced(sre);
+            ManagedListener listener = servletRequestAttributeListeners[i];
+            try {
+                this.<ServletRequestAttributeListener> get(listener).attributeReplaced(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeReplaced", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -314,7 +346,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final HttpSessionEvent sre = new HttpSessionEvent(session);
         for (int i = 0; i < httpSessionListeners.length; ++i) {
-            this.<HttpSessionListener>get(httpSessionListeners[i]).sessionCreated(sre);
+            ManagedListener listener = httpSessionListeners[i];
+            try {
+                this.<HttpSessionListener> get(listener).sessionCreated(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("sessionCreated", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -325,7 +362,11 @@ public class ApplicationListeners implements Lifecycle {
         final HttpSessionEvent sre = new HttpSessionEvent(session);
         for (int i = httpSessionListeners.length - 1; i >= 0; --i) {
             ManagedListener listener = httpSessionListeners[i];
-            this.<HttpSessionListener>get(listener).sessionDestroyed(sre);
+            try {
+                this.<HttpSessionListener> get(listener).sessionDestroyed(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("sessionDestroyed", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -335,7 +376,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final HttpSessionBindingEvent sre = new HttpSessionBindingEvent(session, name, value);
         for (int i = 0; i < httpSessionAttributeListeners.length; ++i) {
-            this.<HttpSessionAttributeListener>get(httpSessionAttributeListeners[i]).attributeAdded(sre);
+            ManagedListener listener = httpSessionAttributeListeners[i];
+            try {
+                this.<HttpSessionAttributeListener> get(listener).attributeAdded(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeAdded", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -345,7 +391,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final HttpSessionBindingEvent sre = new HttpSessionBindingEvent(session, name, value);
         for (int i = 0; i < httpSessionAttributeListeners.length; ++i) {
-            this.<HttpSessionAttributeListener>get(httpSessionAttributeListeners[i]).attributeRemoved(sre);
+            ManagedListener listener = httpSessionAttributeListeners[i];
+            try {
+                this.<HttpSessionAttributeListener> get(httpSessionAttributeListeners[i]).attributeRemoved(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeRemoved", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -355,7 +406,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final HttpSessionBindingEvent sre = new HttpSessionBindingEvent(session, name, value);
         for (int i = 0; i < httpSessionAttributeListeners.length; ++i) {
-            this.<HttpSessionAttributeListener>get(httpSessionAttributeListeners[i]).attributeReplaced(sre);
+            ManagedListener listener = httpSessionAttributeListeners[i];
+            try {
+                this.<HttpSessionAttributeListener> get(listener).attributeReplaced(sre);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("attributeReplaced", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
@@ -365,7 +421,12 @@ public class ApplicationListeners implements Lifecycle {
         }
         final HttpSessionEvent sre = new HttpSessionEvent(session);
         for (int i = 0; i < httpSessionIdListeners.length; ++i) {
-            this.<HttpSessionIdListener>get(httpSessionIdListeners[i]).sessionIdChanged(sre, oldSessionId);
+            ManagedListener listener = httpSessionIdListeners[i];
+            try {
+                this.<HttpSessionIdListener> get(listener).sessionIdChanged(sre, oldSessionId);
+            } catch (Exception e) {
+                UndertowServletLogger.REQUEST_LOGGER.errorInvokingListener("sessionIdChanged", listener.getListenerInfo().getListenerClass(), e);
+            }
         }
     }
 
