@@ -40,6 +40,7 @@ import io.undertow.servlet.spec.HttpServletResponseImpl;
 import io.undertow.servlet.spec.RequestDispatcherImpl;
 import io.undertow.servlet.spec.ServletContextImpl;
 import io.undertow.util.HeaderValues;
+import io.undertow.util.Headers;
 import io.undertow.util.HttpString;
 import io.undertow.util.Protocols;
 import io.undertow.util.StatusCodes;
@@ -56,16 +57,15 @@ import org.xnio.conduits.ConduitStreamSinkChannel;
 import org.xnio.conduits.ConduitStreamSourceChannel;
 import org.xnio.conduits.StreamSinkConduit;
 
-import javax.servlet.DispatcherType;
-import javax.servlet.ServletException;
-import javax.servlet.ServletRequest;
-import javax.servlet.ServletResponse;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.DispatcherType;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.ServletRequest;
+import jakarta.servlet.ServletResponse;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 import java.io.IOException;
 import java.net.SocketAddress;
 import java.nio.ByteBuffer;
-import java.security.AccessController;
 import java.security.PrivilegedExceptionAction;
 import java.util.Map;
 import java.util.concurrent.Executor;
@@ -92,25 +92,21 @@ public class ServletInitialHandler implements HttpHandler, ServletDispatcher {
     private final ServletPathMatches paths;
 
     private final ExceptionHandler exceptionHandler;
-    private final HttpHandler dispatchHandler = new HttpHandler() {
-        @Override
-        public void handleRequest(final HttpServerExchange exchange) throws Exception {
-            final ServletRequestContext servletRequestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
-            if (System.getSecurityManager() == null) {
+    @SuppressWarnings("removal")
+    private final HttpHandler dispatchHandler = exchange -> {
+        final ServletRequestContext servletRequestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
+        if (System.getSecurityManager() == null) {
+            dispatchRequest(exchange, servletRequestContext, servletRequestContext.getOriginalServletPathMatch().getServletChain(), DispatcherType.REQUEST);
+        } else {
+            //sometimes thread pools inherit some random
+            java.security.AccessController.doPrivileged((PrivilegedExceptionAction<Object>) () -> {
                 dispatchRequest(exchange, servletRequestContext, servletRequestContext.getOriginalServletPathMatch().getServletChain(), DispatcherType.REQUEST);
-            } else {
-                //sometimes thread pools inherit some random
-                AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
-                    @Override
-                    public Object run() throws Exception {
-                        dispatchRequest(exchange, servletRequestContext, servletRequestContext.getOriginalServletPathMatch().getServletChain(), DispatcherType.REQUEST);
-                        return null;
-                    }
-                });
-            }
+                return null;
+            });
         }
     };
 
+    @SuppressWarnings("removal")
     public ServletInitialHandler(final ServletPathMatches paths, final HttpHandler next, final Deployment deployment, final ServletContextImpl servletContext) {
         this.next = next;
         this.servletContext = servletContext;
@@ -118,7 +114,7 @@ public class ServletInitialHandler implements HttpHandler, ServletDispatcher {
         this.listeners = servletContext.getDeployment().getApplicationListeners();
         SecurityManager sm = System.getSecurityManager();
         if(sm != null) {
-            //handle request can use doPrivilidged
+            //handle request can use doPrivileged
             //we need to make sure this is not abused
             sm.checkPermission(PERMISSION);
         }
@@ -140,7 +136,7 @@ public class ServletInitialHandler implements HttpHandler, ServletDispatcher {
     @Override
     public void handleRequest(final HttpServerExchange exchange) throws Exception {
         final String path = exchange.getRelativePath();
-        if(isForbiddenPath(path)) {
+        if (Paths.isForbidden(path)) {
             exchange.setStatusCode(StatusCodes.NOT_FOUND);
             return;
         }
@@ -176,11 +172,6 @@ public class ServletInitialHandler implements HttpHandler, ServletDispatcher {
         }
     }
 
-    private boolean isForbiddenPath(String path) {
-        return path.equalsIgnoreCase("/meta-inf/")
-            || path.regionMatches(true, 0, "/web-inf/", 0, "/web-inf/".length());
-    }
-
     public void dispatchToPath(final HttpServerExchange exchange, final ServletPathMatch pathInfo, final DispatcherType dispatcherType) throws Exception {
         final ServletRequestContext servletRequestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
         servletRequestContext.setServletPathMatch(pathInfo);
@@ -243,10 +234,10 @@ public class ServletInitialHandler implements HttpHandler, ServletDispatcher {
         }
     }
 
-    private boolean isMultiPartExchange(final HttpServerExchange exhange) {
+    private boolean isMultiPartExchange(final HttpServerExchange exchange) {
         //NOTE: should this include Range response?
-        final HeaderValues contentTypeHeaders = exhange.getRequestHeaders().get("Content-Type");
-        if(contentTypeHeaders != null && contentTypeHeaders.size() >0) {
+        final HeaderValues contentTypeHeaders = exchange.getRequestHeaders().get(Headers.CONTENT_TYPE);
+        if (contentTypeHeaders != null && !contentTypeHeaders.isEmpty()) {
             return contentTypeHeaders.getFirst().startsWith("multipart");
         } else {
             return false;
