@@ -376,6 +376,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
     }
 
     @Override
+    @SuppressWarnings("removal")
     public String changeSessionId() {
         HttpSessionImpl session = servletContext.getSession(originalServletContext, exchange, false);
         if (session == null) {
@@ -553,7 +554,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
 
     @Override
     public Collection<Part> getParts() throws IOException, ServletException {
-        verifyMultipartServlet();
         if (parts == null) {
             loadParts();
         }
@@ -570,11 +570,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
 
     @Override
     public Part getPart(final String name) throws IOException, ServletException {
-        verifyMultipartServlet();
-        if (parts == null) {
-            loadParts();
-        }
-        for (Part part : parts) {
+        for (Part part : getParts()) {
             if (part.getName().equals(name)) {
                 return part;
             }
@@ -600,6 +596,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         final ServletRequestContext requestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
 
         if (parts == null) {
+            verifyMultipartServlet();
             final List<Part> parts = new ArrayList<>();
             String mimeType = exchange.getRequestHeaders().getFirst(Headers.CONTENT_TYPE);
             if (mimeType != null && mimeType.startsWith(MultiPartParserDefinition.MULTIPART_FORM_DATA)) {
@@ -758,7 +755,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             final FormData parsedFormData = parseFormData();
             if (parsedFormData != null) {
                 FormData.FormValue res = parsedFormData.getFirst(name);
-                if (res == null || res.isFileItem()) {
+                if (res == null || res.isFileItem() && !res.isBigField()) {
                     return null;
                 } else {
                     return res.getValue();
@@ -781,7 +778,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             while (it.hasNext()) {
                 String name = it.next();
                 for(FormData.FormValue param : parsedFormData.get(name)) {
-                    if(!param.isFileItem()) {
+                    if(!param.isFileItem() || param.isBigField()) {
                         parameterNames.add(name);
                         break;
                     }
@@ -808,7 +805,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             Deque<FormData.FormValue> res = parsedFormData.get(name);
             if (res != null) {
                 for (FormData.FormValue value : res) {
-                    if(!value.isFileItem()) {
+                    if(!value.isFileItem() || value.isBigField()) {
                         ret.add(value.getValue());
                     }
                 }
@@ -839,14 +836,14 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
                 if (arrayMap.containsKey(name)) {
                     ArrayList<String> existing = arrayMap.get(name);
                     for (final FormData.FormValue v : val) {
-                        if(!v.isFileItem()) {
+                        if(!v.isFileItem() || v.isBigField()) {
                             existing.add(v.getValue());
                         }
                     }
                 } else {
                     final ArrayList<String> values = new ArrayList<>();
                     for (final FormData.FormValue v : val) {
-                        if(!v.isFileItem()) {
+                        if(!v.isFileItem() || v.isBigField()) {
                             values.add(v.getValue());
                         }
                     }
@@ -1037,7 +1034,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
      * Gets the host name for this IP address.
      * If this InetAddress was created with a host name, this host name will be remembered and returned; otherwise, a reverse name lookup will be performed and the result will be returned based on the system configured name lookup service. If a lookup of the name service is required, call getCanonicalHostName.
      * If there is a security manager, its checkConnect method is first called with the hostname and -1 as its arguments to see if the operation is allowed. If the operation is not allowed, it will return the textual representation of the IP address.
-     * @see InetAddres#getHostName
+     * @see InetAddress#getHostName(boolean)
      */
     @Override
     public String getLocalName() {
@@ -1075,9 +1072,10 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         } else if (asyncStarted) {
             throw UndertowServletMessages.MESSAGES.asyncAlreadyStarted();
         }
-        asyncStarted = true;
         final ServletRequestContext servletRequestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
-        return asyncContext = new AsyncContextImpl(exchange, servletRequestContext.getServletRequest(), servletRequestContext.getServletResponse(), servletRequestContext, false, asyncContext);
+        asyncContext = new AsyncContextImpl(exchange, servletRequestContext.getServletRequest(), servletRequestContext.getServletResponse(), servletRequestContext, false, asyncContext);
+        asyncStarted = true;
+        return asyncContext;
     }
 
     @Override
@@ -1100,10 +1098,11 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         } else if (asyncStarted) {
             throw UndertowServletMessages.MESSAGES.asyncAlreadyStarted();
         }
-        asyncStarted = true;
         servletRequestContext.setServletRequest(servletRequest);
         servletRequestContext.setServletResponse(servletResponse);
-        return asyncContext = new AsyncContextImpl(exchange, servletRequest, servletResponse, servletRequestContext, true, asyncContext);
+        asyncContext = new AsyncContextImpl(exchange, servletRequest, servletResponse, servletRequestContext, true, asyncContext);
+        asyncStarted = true;
+        return asyncContext;
     }
 
     @Override
