@@ -20,7 +20,6 @@ package io.undertow.servlet.spec;
 
 import java.io.IOException;
 import java.io.PrintWriter;
-import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.Deque;
@@ -46,6 +45,7 @@ import io.undertow.servlet.handlers.ServletRequestContext;
 import io.undertow.servlet.handlers.ServletChain;
 import io.undertow.servlet.handlers.ServletPathMatch;
 import io.undertow.servlet.util.DispatchUtils;
+import io.undertow.util.BadRequestException;
 import io.undertow.util.ParameterLimitException;
 
 /**
@@ -74,15 +74,13 @@ public class RequestDispatcherImpl implements RequestDispatcher {
 
 
     @Override
+    @SuppressWarnings("removal")
     public void forward(final ServletRequest request, final ServletResponse response) throws ServletException, IOException {
         if(System.getSecurityManager() != null) {
             try {
-                AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
-                    @Override
-                    public Object run() throws Exception {
-                        forwardImplSetup(request, response);
-                        return null;
-                    }
+                java.security.AccessController.doPrivileged((PrivilegedExceptionAction<Object>) () -> {
+                    forwardImplSetup(request, response);
+                    return null;
                 });
             } catch (PrivilegedActionException e) {
                 if(e.getCause() instanceof ServletException) {
@@ -173,7 +171,7 @@ public class RequestDispatcherImpl implements RequestDispatcher {
         if (!named) {
             try {
                 pathMatch = DispatchUtils.dispatchForward(path, requestImpl, responseImpl, servletContext);
-            } catch (ParameterLimitException e) {
+            } catch (ParameterLimitException | BadRequestException e) {
                 throw new ServletException(e);
             }
         }
@@ -224,15 +222,13 @@ public class RequestDispatcherImpl implements RequestDispatcher {
 
 
     @Override
+    @SuppressWarnings("removal")
     public void include(final ServletRequest request, final ServletResponse response) throws ServletException, IOException {
         if(System.getSecurityManager() != null) {
             try {
-                AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
-                    @Override
-                    public Object run() throws Exception {
-                        setupIncludeImpl(request, response);
-                        return null;
-                    }
+                java.security.AccessController.doPrivileged((PrivilegedExceptionAction<Object>) () -> {
+                    setupIncludeImpl(request, response);
+                    return null;
                 });
             } catch (PrivilegedActionException e) {
                 if(e.getCause() instanceof ServletException) {
@@ -319,7 +315,7 @@ public class RequestDispatcherImpl implements RequestDispatcher {
 
             try {
                 pathMatch = DispatchUtils.dispatchInclude(path, requestImpl, responseImpl, servletContext);
-            } catch (ParameterLimitException e) {
+            } catch (ParameterLimitException | BadRequestException e) {
                 throw new ServletException(e);
             }
         }
@@ -403,7 +399,7 @@ public class RequestDispatcherImpl implements RequestDispatcher {
         ServletPathMatch pathMatch;
         try {
             pathMatch = DispatchUtils.dispatchError(path, servletName, exception, message, requestImpl, responseImpl, servletContext);
-        } catch (ParameterLimitException e) {
+        } catch (ParameterLimitException | BadRequestException e) {
             throw new ServletException(e);
         }
 
