@@ -35,12 +35,12 @@ import io.undertow.servlet.UndertowServletMessages;
  */
 public class ServletCookieAdaptor implements Cookie {
 
-    private final javax.servlet.http.Cookie cookie;
+    private final jakarta.servlet.http.Cookie cookie;
 
     private boolean sameSite;
     private String sameSiteMode;
 
-    public ServletCookieAdaptor(final javax.servlet.http.Cookie cookie) {
+    public ServletCookieAdaptor(final jakarta.servlet.http.Cookie cookie) {
         this.cookie = cookie;
     }
 
@@ -198,8 +198,8 @@ public class ServletCookieAdaptor implements Cookie {
     @Override
     public final boolean equals(final Object other) {
         if (other == this) return true;
-        if (!(other instanceof ServletCookieAdaptor)) return false;
-        final ServletCookieAdaptor o = (ServletCookieAdaptor) other;
+        if (!(other instanceof Cookie)) return false;
+        final Cookie o = (Cookie) other;
         // compare names
         if (getName() == null && o.getName() != null) return false;
         if (getName() != null && !getName().equals(o.getName())) return false;
@@ -213,6 +213,11 @@ public class ServletCookieAdaptor implements Cookie {
         return true;
     }
 
+    @Override
+    public final int compareTo(final Object other) {
+        return Cookie.super.compareTo(other);
+    }
+
     @Override
     public final String toString() {
         return "{ServletCookieAdaptor@" + System.identityHashCode(this) + " name=" + getName() + " path=" + getPath() + " domain=" + getDomain() + "}";
