@@ -20,23 +20,24 @@ package io.undertow.servlet.test.async;
 
 import java.io.IOException;
 
-import javax.servlet.AsyncContext;
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.AsyncContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.HttpServlet;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 
 public class AsyncDispatchServlet extends HttpServlet {
     private static final long serialVersionUID = 1L;
 
     @Override
     protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        final String path = req.getHeader("dispatch") != null? req.getHeader("dispatch") : "/message";
         final AsyncContext ac = req.startAsync(req, new TestAsyncRespWrapper(resp));
         ac.start(new Runnable() {
             @Override
             public void run() {
-                ac.dispatch("/message");
+                ac.dispatch(path);
             }
         });
     }
-}
\ No newline at end of file
+}
