@@ -21,8 +21,8 @@ package io.undertow.servlet.test.async;
 import java.io.IOException;
 import java.io.PrintWriter;
 
-import javax.servlet.http.HttpServletResponse;
-import javax.servlet.http.HttpServletResponseWrapper;
+import jakarta.servlet.http.HttpServletResponse;
+import jakarta.servlet.http.HttpServletResponseWrapper;
 
 class TestAsyncRespWrapper extends HttpServletResponseWrapper {
     TestAsyncRespWrapper(HttpServletResponse resp) {
@@ -35,4 +35,4 @@ class TestAsyncRespWrapper extends HttpServletResponseWrapper {
         writer.write("wrapped: ");
         return writer;
     }
-}
\ No newline at end of file
+}
