@@ -21,13 +21,13 @@ package io.undertow.servlet.test.streams;
 import java.io.IOException;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import javax.servlet.AsyncContext;
-import javax.servlet.ServletException;
-import javax.servlet.ServletOutputStream;
-import javax.servlet.WriteListener;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.AsyncContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.ServletOutputStream;
+import jakarta.servlet.WriteListener;
+import jakarta.servlet.http.HttpServlet;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 
 /**
  * @author Stuart Douglas
@@ -45,6 +45,7 @@ public class AsyncOutputStreamServlet extends HttpServlet {
         final AtomicInteger count = new AtomicInteger();
 
         final AsyncContext context = req.startAsync();
+        context.setTimeout(60000);
         final ServletOutputStream outputStream = resp.getOutputStream();
         if(preable) {
             for(int i = 0; i < reps; ++i) {
