@@ -20,8 +20,8 @@ package io.undertow.websockets.jsr;
 import io.undertow.websockets.core.WebSocketCallback;
 import io.undertow.websockets.core.WebSocketChannel;
 
-import javax.websocket.SendHandler;
-import javax.websocket.SendResult;
+import jakarta.websocket.SendHandler;
+import jakarta.websocket.SendResult;
 
 /**
  * {@link WebSocketCallback} implementation which will notify a wrapped {@link SendHandler} once a send operation
