@@ -53,21 +53,21 @@ import org.xnio.http.UpgradeFailedException;
 import org.xnio.ssl.XnioSsl;
 
 import javax.net.ssl.SSLContext;
-import javax.servlet.DispatcherType;
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
-import javax.websocket.ClientEndpoint;
-import javax.websocket.ClientEndpointConfig;
-import javax.websocket.CloseReason;
-import javax.websocket.DeploymentException;
-import javax.websocket.Endpoint;
-import javax.websocket.Extension;
-import javax.websocket.HandshakeResponse;
-import javax.websocket.Session;
-import javax.websocket.server.ServerContainer;
-import javax.websocket.server.ServerEndpoint;
-import javax.websocket.server.ServerEndpointConfig;
+import jakarta.servlet.DispatcherType;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
+import jakarta.websocket.ClientEndpoint;
+import jakarta.websocket.ClientEndpointConfig;
+import jakarta.websocket.CloseReason;
+import jakarta.websocket.DeploymentException;
+import jakarta.websocket.Endpoint;
+import jakarta.websocket.Extension;
+import jakarta.websocket.HandshakeResponse;
+import jakarta.websocket.Session;
+import jakarta.websocket.server.ServerContainer;
+import jakarta.websocket.server.ServerEndpoint;
+import jakarta.websocket.server.ServerEndpointConfig;
 import java.io.Closeable;
 import java.io.IOException;
 import java.net.InetSocketAddress;
@@ -87,6 +87,7 @@ import java.util.TreeMap;
 import java.util.TreeSet;
 import java.util.concurrent.Executor;
 import java.util.concurrent.TimeUnit;
+import java.util.function.Supplier;
 
 import static java.lang.System.*;
 
@@ -106,6 +107,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
     private final Map<Class<?>, ConfiguredClientEndpoint> clientEndpoints = new CopyOnWriteMap<>();
 
     private final List<ConfiguredServerEndpoint> configuredServerEndpoints = new ArrayList<>();
+    private final Set<Class<?>> annotatedEndpointClasses = new HashSet<>();
 
     /**
      * set of all deployed server endpoint paths. Due to the comparison function we can detect
@@ -113,7 +115,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
      */
     private final TreeSet<PathTemplate> seenPaths = new TreeSet<>();
 
-    private final XnioWorker xnioWorker;
+    private final Supplier<XnioWorker> xnioWorker;
     private final ByteBufferPool bufferPool;
     private final boolean dispatchToWorker;
     private final InetSocketAddress clientBindAddress;
@@ -124,6 +126,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
     private volatile int defaultMaxBinaryMessageBufferSize;
     private volatile int defaultMaxTextMessageBufferSize;
     private volatile boolean deploymentComplete = false;
+    private final List<DeploymentException> deploymentExceptions = new ArrayList<>();
 
     private ServletContextImpl contextToAddFilter = null;
 
@@ -135,19 +138,19 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
 
     private volatile boolean closed = false;
 
-    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final XnioWorker xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, boolean clientMode) {
+    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final Supplier<XnioWorker> xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, boolean clientMode) {
         this(classIntrospecter, ServerWebSocketContainer.class.getClassLoader(), xnioWorker, bufferPool, threadSetupHandlers, dispatchToWorker, null, null);
     }
 
-    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, XnioWorker xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker) {
+    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, Supplier<XnioWorker> xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker) {
         this(classIntrospecter, classLoader, xnioWorker, bufferPool, threadSetupHandlers, dispatchToWorker, null, null);
     }
 
-    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, XnioWorker xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, InetSocketAddress clientBindAddress, WebSocketReconnectHandler reconnectHandler) {
-        this(classIntrospecter, classLoader, xnioWorker, bufferPool, threadSetupHandlers, dispatchToWorker, clientBindAddress, reconnectHandler, Collections.<Extension>emptyList());
+    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, Supplier<XnioWorker> xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, InetSocketAddress clientBindAddress, WebSocketReconnectHandler reconnectHandler) {
+        this(classIntrospecter, classLoader, xnioWorker, bufferPool, threadSetupHandlers, dispatchToWorker, clientBindAddress, reconnectHandler, Collections.emptyList());
     }
 
-    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, XnioWorker xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, InetSocketAddress clientBindAddress, WebSocketReconnectHandler reconnectHandler, List<Extension> installedExtensions) {
+    public ServerWebSocketContainer(final ClassIntrospecter classIntrospecter, final ClassLoader classLoader, Supplier<XnioWorker> xnioWorker, ByteBufferPool bufferPool, List<ThreadSetupHandler> threadSetupHandlers, boolean dispatchToWorker, InetSocketAddress clientBindAddress, WebSocketReconnectHandler reconnectHandler, List<Extension> installedExtensions) {
         this.classIntrospecter = classIntrospecter;
         this.bufferPool = bufferPool;
         this.xnioWorker = xnioWorker;
@@ -208,14 +211,14 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         Endpoint instance = config.getFactory().createInstance(new ImmediateInstanceHandle<>(annotatedEndpointInstance));
         XnioSsl ssl = null;
         for (WebsocketClientSslProvider provider : clientSslProviders) {
-            ssl = provider.getSsl(xnioWorker, annotatedEndpointInstance, path);
+            ssl = provider.getSsl(xnioWorker.get(), annotatedEndpointInstance, path);
             if (ssl != null) {
                 break;
             }
         }
         if(ssl == null) {
             try {
-                ssl = new UndertowXnioSsl(xnioWorker.getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
+                ssl = new UndertowXnioSsl(xnioWorker.get().getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
             } catch (NoSuchAlgorithmException e) {
                 //ignore
             }
@@ -256,14 +259,14 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
             InstanceHandle<?> instance = config.getInstanceFactory().createInstance();
             XnioSsl ssl = null;
             for (WebsocketClientSslProvider provider : clientSslProviders) {
-                ssl = provider.getSsl(xnioWorker, aClass, uri);
+                ssl = provider.getSsl(xnioWorker.get(), aClass, uri);
                 if (ssl != null) {
                     break;
                 }
             }
             if(ssl == null) {
                 try {
-                    ssl = new UndertowXnioSsl(xnioWorker.getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
+                    ssl = new UndertowXnioSsl(xnioWorker.get().getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
                 } catch (NoSuchAlgorithmException e) {
                     //ignore
                 }
@@ -282,14 +285,14 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         ClientEndpointConfig cec = config != null ? config : ClientEndpointConfig.Builder.create().build();
         XnioSsl ssl = null;
         for (WebsocketClientSslProvider provider : clientSslProviders) {
-            ssl = provider.getSsl(xnioWorker, endpointInstance, cec, path);
+            ssl = provider.getSsl(xnioWorker.get(), endpointInstance, cec, path);
             if (ssl != null) {
                 break;
             }
         }
         if(ssl == null) {
             try {
-                ssl = new UndertowXnioSsl(xnioWorker.getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
+                ssl = new UndertowXnioSsl(xnioWorker.get().getXnio(), OptionMap.EMPTY, SSLContext.getDefault());
             } catch (NoSuchAlgorithmException e) {
                 //ignore
             }
@@ -298,7 +301,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         WebSocketClientNegotiation clientNegotiation = new ClientNegotiation(cec.getPreferredSubprotocols(), toExtensionList(cec.getExtensions()), cec);
 
 
-        WebSocketClient.ConnectionBuilder connectionBuilder = WebSocketClient.connectionBuilder(xnioWorker, bufferPool, path)
+        WebSocketClient.ConnectionBuilder connectionBuilder = WebSocketClient.connectionBuilder(xnioWorker.get(), bufferPool, path)
                 .setSsl(ssl)
                 .setBindAddress(clientBindAddress)
                 .setClientNegotiation(clientNegotiation);
@@ -349,18 +352,24 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
             extensions.add(ExtensionImpl.create(e));
         }
         ConfiguredClientEndpoint configured = clientEndpoints.get(endpointInstance.getClass());
+        Endpoint instance = endpointInstance;
         if(configured == null) {
             synchronized (clientEndpoints) {
-                configured = clientEndpoints.get(endpointInstance.getClass());
+                // make sure to create an instance of AnnotatedEndpoint if we have the annotation
+                configured = getClientEndpoint(endpointInstance.getClass(), false);
                 if(configured == null) {
+                    // if we don't, add an endpoint anyway to the list of clientEndpoints
                     clientEndpoints.put(endpointInstance.getClass(), configured = new ConfiguredClientEndpoint());
+                } else {
+                    // use the  factory in configured to reach the endpoint
+                    instance = configured.getFactory().createInstance(new ImmediateInstanceHandle<>(endpointInstance));
                 }
             }
         }
 
         EncodingFactory encodingFactory = EncodingFactory.createFactory(classIntrospecter, cec.getDecoders(), cec.getEncoders());
         UndertowSession undertowSession = new UndertowSession(channel, connectionBuilder.getUri(), Collections.<String, String>emptyMap(), Collections.<String, List<String>>emptyMap(), sessionHandler, null, new ImmediateInstanceHandle<>(endpointInstance), cec, connectionBuilder.getUri().getQuery(), encodingFactory.createEncoding(cec), configured, clientNegotiation.getSelectedSubProtocol(), extensions, connectionBuilder);
-        endpointInstance.onOpen(undertowSession, cec);
+        instance.onOpen(undertowSession, cec);
         channel.resumeReceives();
 
         return undertowSession;
@@ -380,6 +389,14 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         }
     }
 
+    @Override
+    public void upgradeHttpToWebSocket(Object req, Object res, ServerEndpointConfig sec, Map<String, String> pathParameters) throws IOException, DeploymentException {
+        try {
+            doUpgrade((HttpServletRequest) req, (HttpServletResponse) res, sec, pathParameters);
+        } catch (final ServletException e) {
+            throw new DeploymentException(e.getRootCause().getMessage(), e.getRootCause());
+        }
+    }
 
     public void doUpgrade(HttpServletRequest request,
                           HttpServletResponse response, final ServerEndpointConfig sec,
@@ -440,7 +457,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
                 hand = ServerWebSocketContainer.handshakes(confguredServerEndpoint, info.getExtensions());
             }
 
-            final ServletWebSocketHttpExchange facade = new ServletWebSocketHttpExchange(request, response, new HashSet<WebSocketChannel>());
+            final ServletWebSocketHttpExchange facade = new ServletWebSocketHttpExchange(request, response, new HashSet<>());
             Handshake handshaker = null;
             for (Handshake method : hand.handshakes) {
                 if (method.matches(facade)) {
@@ -476,7 +493,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         WebSocketClientNegotiation clientNegotiation = new ClientNegotiation(cec.getConfig().getPreferredSubprotocols(), toExtensionList(cec.getConfig().getExtensions()), cec.getConfig());
 
 
-        WebSocketClient.ConnectionBuilder connectionBuilder = WebSocketClient.connectionBuilder(xnioWorker, bufferPool, path)
+        WebSocketClient.ConnectionBuilder connectionBuilder = WebSocketClient.connectionBuilder(xnioWorker.get(), bufferPool, path)
                 .setSsl(ssl)
                 .setBindAddress(clientBindAddress)
                 .setClientNegotiation(clientNegotiation);
@@ -609,7 +626,18 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         if (deploymentComplete) {
             throw JsrWebSocketMessages.MESSAGES.cannotAddEndpointAfterDeployment();
         }
-        addEndpointInternal(endpoint, true);
+        //work around a TCK7 problem
+        //if the class has already been added we just ignore it
+        if(annotatedEndpointClasses.contains(endpoint)) {
+            return;
+        }
+        annotatedEndpointClasses.add(endpoint);
+        try {
+            addEndpointInternal(endpoint, true);
+        } catch (DeploymentException e) {
+            deploymentExceptions.add(e);
+            throw e;
+        }
     }
 
     private synchronized void addEndpointInternal(final Class<?> endpoint, boolean requiresCreation) throws DeploymentException {
@@ -744,7 +772,13 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
         }
         seenPaths.add(template);
         EncodingFactory encodingFactory = EncodingFactory.createFactory(classIntrospecter, endpoint.getDecoders(), endpoint.getEncoders());
-        ConfiguredServerEndpoint confguredServerEndpoint = new ConfiguredServerEndpoint(endpoint, null, template, encodingFactory);
+
+        AnnotatedEndpointFactory annotatedEndpointFactory = null;
+        if(!Endpoint.class.isAssignableFrom(endpoint.getEndpointClass())) {
+            // We may want to check that the path in @ServerEndpoint matches the specified path, and throw if they are not equivalent
+            annotatedEndpointFactory = AnnotatedEndpointFactory.create(endpoint.getEndpointClass(), encodingFactory, template.getParameterNames());
+        }
+        ConfiguredServerEndpoint confguredServerEndpoint = new ConfiguredServerEndpoint(endpoint, null, template, encodingFactory, annotatedEndpointFactory, endpoint.getExtensions());
         configuredServerEndpoints.add(confguredServerEndpoint);
         handleAddingFilterMapping();
     }
@@ -781,8 +815,20 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
     }
 
 
+
+    public void validateDeployment() {
+        if(!deploymentExceptions.isEmpty()) {
+            RuntimeException e = JsrWebSocketMessages.MESSAGES.deploymentFailedDueToProgramaticErrors();
+            for(DeploymentException ex : deploymentExceptions) {
+                e.addSuppressed(ex);
+            }
+            throw e;
+        }
+    }
+
     public void deploymentComplete() {
         deploymentComplete = true;
+        validateDeployment();
     }
 
     public List<ConfiguredServerEndpoint> getConfiguredServerEndpoints() {
@@ -815,7 +861,7 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
     }
 
     public XnioWorker getXnioWorker() {
-        return xnioWorker;
+        return xnioWorker.get();
     }
 
     private static List<WebSocketExtension> toExtensionList(final List<Extension> extensions) {
@@ -914,15 +960,19 @@ public class ServerWebSocketContainer implements ServerContainer, Closeable {
 
             @Override
             public synchronized void run() {
+                List<PauseListener> copy = null;
                 synchronized (ServerWebSocketContainer.this) {
                     count--;
                     if (count == 0) {
-                        for(PauseListener p : pauseListeners) {
-                            p.paused();
-                        }
+                        copy = new ArrayList<>(pauseListeners);
                         pauseListeners.clear();
                     }
                 }
+                if(copy != null) {
+                    for (PauseListener p : copy) {
+                        p.paused();
+                    }
+                }
             }
         };
 
