@@ -18,10 +18,10 @@
 
 package io.undertow.websockets.jsr.test.dynamicupgrade;
 
-import javax.websocket.OnMessage;
-import javax.websocket.OnOpen;
-import javax.websocket.Session;
-import javax.websocket.server.PathParam;
+import jakarta.websocket.OnMessage;
+import jakarta.websocket.OnOpen;
+import jakarta.websocket.Session;
+import jakarta.websocket.server.PathParam;
 
 /**
  * @author Stuart Douglas
